/* SPDX-License-Identifier: (LGPL-2.1 OR BSD-2-Clause) */

/* THIS FILE IS AUTOGENERATED BY BPFTOOL! */
#ifndef __SYSCALL_SKEL_H__
#define __SYSCALL_SKEL_H__

#include <errno.h>
#include <stdlib.h>
#include <bpf/libbpf.h>

struct syscall {
	struct bpf_object_skeleton *skeleton;
	struct bpf_object *obj;
	struct {
		struct bpf_map *inner_map;
		struct bpf_map *outer_array_map;
		struct bpf_map *bpf_attr_array;
		struct bpf_map *data;
		struct bpf_map *rodata;
	} maps;
	struct {
		struct bpf_program *load_prog;
		struct bpf_program *update_outer_map;
	} progs;
	struct {
		struct bpf_link *load_prog;
		struct bpf_link *update_outer_map;
	} links;

#ifdef __cplusplus
	static inline struct syscall *open(const struct bpf_object_open_opts *opts = nullptr);
	static inline struct syscall *open_and_load();
	static inline int load(struct syscall *skel);
	static inline int attach(struct syscall *skel);
	static inline void detach(struct syscall *skel);
	static inline void destroy(struct syscall *skel);
	static inline const void *elf_bytes(size_t *sz);
#endif /* __cplusplus */
};

static void
syscall__destroy(struct syscall *obj)
{
	if (!obj)
		return;
	if (obj->skeleton)
		bpf_object__destroy_skeleton(obj->skeleton);
	free(obj);
}

static inline int
syscall__create_skeleton(struct syscall *obj);

static inline struct syscall *
syscall__open_opts(const struct bpf_object_open_opts *opts)
{
	struct syscall *obj;
	int err;

	obj = (struct syscall *)calloc(1, sizeof(*obj));
	if (!obj) {
		errno = ENOMEM;
		return NULL;
	}

	err = syscall__create_skeleton(obj);
	if (err)
		goto err_out;

	err = bpf_object__open_skeleton(obj->skeleton, opts);
	if (err)
		goto err_out;

	return obj;
err_out:
	syscall__destroy(obj);
	errno = -err;
	return NULL;
}

static inline struct syscall *
syscall__open(void)
{
	return syscall__open_opts(NULL);
}

static inline int
syscall__load(struct syscall *obj)
{
	return bpf_object__load_skeleton(obj->skeleton);
}

static inline struct syscall *
syscall__open_and_load(void)
{
	struct syscall *obj;
	int err;

	obj = syscall__open();
	if (!obj)
		return NULL;
	err = syscall__load(obj);
	if (err) {
		syscall__destroy(obj);
		errno = -err;
		return NULL;
	}
	return obj;
}

static inline int
syscall__attach(struct syscall *obj)
{
	return bpf_object__attach_skeleton(obj->skeleton);
}

static inline void
syscall__detach(struct syscall *obj)
{
	bpf_object__detach_skeleton(obj->skeleton);
}

static inline const void *syscall__elf_bytes(size_t *sz);

static inline int
syscall__create_skeleton(struct syscall *obj)
{
	struct bpf_object_skeleton *s;
	int err;

	s = (struct bpf_object_skeleton *)calloc(1, sizeof(*s));
	if (!s)	{
		err = -ENOMEM;
		goto err;
	}

	s->sz = sizeof(*s);
	s->name = "syscall";
	s->obj = &obj->obj;

	/* maps */
	s->map_cnt = 5;
	s->map_skel_sz = sizeof(*s->maps);
	s->maps = (struct bpf_map_skeleton *)calloc(s->map_cnt, s->map_skel_sz);
	if (!s->maps) {
		err = -ENOMEM;
		goto err;
	}

	s->maps[0].name = "inner_map";
	s->maps[0].map = &obj->maps.inner_map;

	s->maps[1].name = "outer_array_map";
	s->maps[1].map = &obj->maps.outer_array_map;

	s->maps[2].name = "bpf_attr_array";
	s->maps[2].map = &obj->maps.bpf_attr_array;

	s->maps[3].name = "syscall.data";
	s->maps[3].map = &obj->maps.data;

	s->maps[4].name = "syscall.rodata";
	s->maps[4].map = &obj->maps.rodata;

	/* programs */
	s->prog_cnt = 2;
	s->prog_skel_sz = sizeof(*s->progs);
	s->progs = (struct bpf_prog_skeleton *)calloc(s->prog_cnt, s->prog_skel_sz);
	if (!s->progs) {
		err = -ENOMEM;
		goto err;
	}

	s->progs[0].name = "load_prog";
	s->progs[0].prog = &obj->progs.load_prog;
	s->progs[0].link = &obj->links.load_prog;

	s->progs[1].name = "update_outer_map";
	s->progs[1].prog = &obj->progs.update_outer_map;
	s->progs[1].link = &obj->links.update_outer_map;

	s->data = syscall__elf_bytes(&s->data_sz);

	obj->skeleton = s;
	return 0;
err:
	bpf_object__destroy_skeleton(s);
	return err;
}

static inline const void *syscall__elf_bytes(size_t *sz)
{
	static const char data[] __attribute__((__aligned__(8))) = "\
\x7f\x45\x4c\x46\x02\x01\x01\0\0\0\0\0\0\0\0\0\x01\0\xf7\0\x01\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x88\x32\0\0\0\0\0\0\0\0\0\0\x40\0\0\0\0\0\x40\0\x0c\0\
\x01\0\0\x2e\x73\x74\x72\x74\x61\x62\0\x2e\x73\x79\x6d\x74\x61\x62\0\x73\x79\
\x73\x63\x61\x6c\x6c\0\x6c\x69\x63\x65\x6e\x73\x65\0\x2e\x6d\x61\x70\x73\0\x2e\
\x64\x61\x74\x61\0\x2e\x72\x6f\x64\x61\x74\x61\0\x73\x79\x73\x63\x61\x6c\x6c\
\x2e\x63\0\x62\x74\x66\x5f\x6c\x6f\x61\x64\x2e\x62\x74\x66\x5f\x6c\x6f\x61\x64\
\x5f\x61\x74\x74\x72\0\x4c\x42\x42\x30\x5f\x35\0\x6c\x6f\x61\x64\x5f\x70\x72\
\x6f\x67\x2e\x6d\x61\x70\x5f\x63\x72\x65\x61\x74\x65\x5f\x61\x74\x74\x72\0\x6c\
\x6f\x61\x64\x5f\x70\x72\x6f\x67\x2e\x70\x72\x6f\x67\x5f\x6c\x6f\x61\x64\x5f\
\x61\x74\x74\x72\0\x6c\x6f\x61\x64\x5f\x70\x72\x6f\x67\x2e\x6c\x69\x63\x65\x6e\
\x73\x65\0\x6c\x6f\x61\x64\x5f\x70\x72\x6f\x67\x2e\x69\x6e\x73\x6e\x73\0\x6c\
\x6f\x61\x64\x5f\x70\x72\x6f\x67\x2e\x6d\x61\x70\x5f\x75\x70\x64\x61\x74\x65\
\x5f\x61\x74\x74\x72\0\x6c\x6f\x61\x64\x5f\x70\x72\x6f\x67\x2e\x76\x61\x6c\x75\
\x65\0\x6c\x6f\x61\x64\x5f\x70\x72\x6f\x67\x2e\x6b\x65\x79\0\x4c\x42\x42\x31\
\x5f\x36\0\x4c\x42\x42\x31\x5f\x39\0\x4c\x42\x42\x31\x5f\x38\0\x4c\x42\x42\x31\
\x5f\x31\x30\0\x6c\x6f\x61\x64\x5f\x70\x72\x6f\x67\0\x75\x70\x64\x61\x74\x65\
\x5f\x6f\x75\x74\x65\x72\x5f\x6d\x61\x70\0\x62\x70\x66\x5f\x61\x74\x74\x72\x5f\
\x61\x72\x72\x61\x79\0\x6f\x75\x74\x65\x72\x5f\x61\x72\x72\x61\x79\x5f\x6d\x61\
\x70\0\x5f\x6c\x69\x63\x65\x6e\x73\x65\0\x69\x6e\x6e\x65\x72\x5f\x6d\x61\x70\0\
\x2e\x72\x65\x6c\x73\x79\x73\x63\x61\x6c\x6c\0\x2e\x72\x65\x6c\x2e\x6d\x61\x70\
\x73\0\x2e\x42\x54\x46\0\x2e\x42\x54\x46\x2e\x65\x78\x74\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x35\0\0\0\x04\0\xf1\xff\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\x03\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x3f\0\0\0\x01\0\x06\
\0\x08\x02\0\0\0\0\0\0\x90\0\0\0\0\0\0\0\x56\0\0\0\0\0\x03\0\x90\x02\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\x5d\0\0\0\x01\0\x06\0\x48\0\0\0\0\0\0\0\x90\0\0\0\0\0\0\0\
\x77\0\0\0\x01\0\x06\0\x78\x01\0\0\0\0\0\0\x90\0\0\0\0\0\0\0\x90\0\0\0\x01\0\
\x06\0\0\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\xa2\0\0\0\x01\0\x06\0\x04\0\0\0\0\0\0\
\0\x40\0\0\0\0\0\0\0\xb2\0\0\0\x01\0\x06\0\xd8\0\0\0\0\0\0\0\x90\0\0\0\0\0\0\0\
\xcc\0\0\0\x01\0\x06\0\x70\x01\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\xdc\0\0\0\x01\0\
\x06\0\x68\x01\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\xea\0\0\0\0\0\x03\0\x80\x06\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\xf1\0\0\0\0\0\x03\0\xa0\x06\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\xf8\0\0\0\0\0\x03\0\x98\x06\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xff\0\0\0\0\0\x03\0\
\xb0\x06\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x03\0\x06\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\x07\x01\0\0\x12\0\x03\0\0\0\0\0\0\0\0\0\x98\x02\0\0\0\0\0\0\x11\x01\
\0\0\x12\0\x03\0\x98\x02\0\0\0\0\0\0\x28\x04\0\0\0\0\0\0\x22\x01\0\0\x11\0\x05\
\0\x48\0\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\x31\x01\0\0\x11\0\x05\0\x20\0\0\0\0\0\0\
\0\x28\0\0\0\0\0\0\0\x41\x01\0\0\x11\0\x04\0\0\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\
\x4a\x01\0\0\x11\0\x05\0\0\0\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\xbf\x16\0\0\0\0\0\0\
\x18\x01\0\0\x08\x02\0\0\0\0\0\0\0\0\0\0\xbf\xa2\0\0\0\0\0\0\x07\x02\0\0\xc0\
\xff\xff\xff\x7b\x21\0\0\0\0\0\0\x18\x01\0\0\x08\0\0\0\0\0\0\0\x40\0\0\0\x7b\
\x1a\xf0\xff\0\0\0\0\x18\x01\0\0\x08\0\0\0\0\0\0\0\x40\0\0\x01\x7b\x1a\xe0\xff\
\0\0\0\0\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\x7b\x1a\xe8\xff\0\0\0\0\x7b\x1a\
\xd8\xff\0\0\0\0\x18\x01\0\0\x20\0\0\0\0\0\0\0\x04\0\0\0\x7b\x1a\xd0\xff\0\0\0\
\0\x18\x01\0\0\0\0\0\0\0\0\0\0\x20\0\0\0\x7b\x1a\xc8\xff\0\0\0\0\x18\x01\0\0\
\x9f\xeb\x01\0\0\0\0\0\x18\0\0\0\x7b\x1a\xc0\xff\0\0\0\0\x62\x0a\xf8\xff\0\0\0\
\0\xb4\x01\0\0\x12\0\0\0\x18\x02\0\0\x08\x02\0\0\0\0\0\0\0\0\0\0\xb4\x03\0\0\
\x90\0\0\0\x85\0\0\0\xa6\0\0\0\xc6\0\x32\0\x01\0\0\0\x63\x06\x18\0\0\0\0\0\x61\
\x61\x0c\0\0\0\0\0\x18\x02\0\0\x48\0\0\0\0\0\0\0\0\0\0\0\x63\x02\x30\0\0\0\0\0\
\x63\x12\x0c\0\0\0\0\0\x18\x01\0\0\x78\x01\0\0\0\0\0\0\0\0\0\0\x18\x02\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\x7b\x21\x10\0\0\0\0\0\x18\x07\0\0\x04\0\0\0\0\0\0\0\0\0\0\
\0\x7b\x71\x08\0\0\0\0\0\x79\x62\0\0\0\0\0\0\x7b\x21\x20\0\0\0\0\0\x61\x62\x08\
\0\0\0\0\0\x63\x21\x1c\0\0\0\0\0\x62\x01\x18\0\x01\0\0\0\xb4\x01\0\0\0\0\0\0\
\x18\x02\0\0\x48\0\0\0\0\0\0\0\0\0\0\0\xb4\x03\0\0\x90\0\0\0\x85\0\0\0\xa6\0\0\
\0\xc6\0\x19\0\x01\0\0\0\x63\x06\x10\0\0\0\0\0\x18\x01\0\0\xd8\0\0\0\0\0\0\0\0\
\0\0\0\x18\x02\0\0\x70\x01\0\0\0\0\0\0\0\0\0\0\x7b\x21\x10\0\0\0\0\0\x18\x02\0\
\0\x68\x01\0\0\0\0\0\0\0\0\0\0\x7b\x21\x08\0\0\0\0\0\x63\x07\x1c\0\0\0\0\0\x63\
\x01\0\0\0\0\0\0\xb4\x01\0\0\x02\0\0\0\x18\x02\0\0\xd8\0\0\0\0\0\0\0\0\0\0\0\
\xb4\x03\0\0\x90\0\0\0\x85\0\0\0\xa6\0\0\0\xc6\0\x08\0\0\0\0\0\xb4\x01\0\0\x05\
\0\0\0\x18\x02\0\0\x78\x01\0\0\0\0\0\0\0\0\0\0\xb4\x03\0\0\x90\0\0\0\x85\0\0\0\
\xa6\0\0\0\xc6\0\x02\0\x01\0\0\0\x63\x06\x14\0\0\0\0\0\xb4\0\0\0\x01\0\0\0\x95\
\0\0\0\0\0\0\0\x62\x0a\xfc\xff\0\0\0\0\x62\x0a\xf8\xff\xff\xff\xff\xff\xbf\xa2\
\0\0\0\0\0\0\x07\x02\0\0\xfc\xff\xff\xff\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\x85\0\0\0\x01\0\0\0\xbf\x07\0\0\0\0\0\0\xb4\x08\0\0\0\0\0\0\xb4\x06\0\0\xff\
\xff\xff\xff\x15\x07\x72\0\0\0\0\0\x7a\x07\x88\0\0\0\0\0\x7a\x07\x80\0\0\0\0\0\
\x7a\x07\x78\0\0\0\0\0\x7a\x07\x70\0\0\0\0\0\x7a\x07\x68\0\0\0\0\0\x7a\x07\x60\
\0\0\0\0\0\x7a\x07\x58\0\0\0\0\0\x7a\x07\x50\0\0\0\0\0\x7a\x07\x48\0\0\0\0\0\
\x7a\x07\x40\0\0\0\0\0\x7a\x07\x38\0\0\0\0\0\x7a\x07\x30\0\0\0\0\0\x7a\x07\x28\
\0\0\0\0\0\x7a\x07\x20\0\0\0\0\0\x7a\x07\x18\0\0\0\0\0\x7a\x07\x10\0\0\0\0\0\
\x7a\x07\x08\0\0\0\0\0\x7a\x07\0\0\0\0\0\0\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\x61\x11\0\0\0\0\0\0\x63\x17\0\0\0\0\0\0\xb4\x01\0\0\x0e\0\0\0\xbf\x72\0\0\0\0\
\0\0\xb4\x03\0\0\x90\0\0\0\x85\0\0\0\xa6\0\0\0\xbf\x06\0\0\0\0\0\0\xc6\x06\x56\
\0\0\0\0\0\x18\x01\0\0\x04\0\0\0\0\0\0\0\x01\0\0\0\x7b\x17\x08\0\0\0\0\0\x18\
\x01\0\0\x02\0\0\0\0\0\0\0\x04\0\0\0\x7b\x17\0\0\0\0\0\0\x7a\x07\x88\0\0\0\0\0\
\x7a\x07\x80\0\0\0\0\0\x7a\x07\x78\0\0\0\0\0\x7a\x07\x70\0\0\0\0\0\x7a\x07\x68\
\0\0\0\0\0\x7a\x07\x60\0\0\0\0\0\x7a\x07\x58\0\0\0\0\0\x7a\x07\x50\0\0\0\0\0\
\x7a\x07\x48\0\0\0\0\0\x7a\x07\x40\0\0\0\0\0\x7a\x07\x38\0\0\0\0\0\x7a\x07\x30\
\0\0\0\0\0\x7a\x07\x28\0\0\0\0\0\x7a\x07\x20\0\0\0\0\0\x7a\x07\x18\0\0\0\0\0\
\x7a\x07\x10\0\0\0\0\0\xb4\x01\0\0\0\0\0\0\xbf\x72\0\0\0\0\0\0\xb4\x03\0\0\x90\
\0\0\0\x85\0\0\0\xa6\0\0\0\x63\x0a\xf8\xff\0\0\0\0\xc6\0\x3e\0\0\0\0\0\x7a\x07\
\x88\0\0\0\0\0\x7a\x07\x80\0\0\0\0\0\x7a\x07\x78\0\0\0\0\0\x7a\x07\x70\0\0\0\0\
\0\x7a\x07\x68\0\0\0\0\0\x7a\x07\x60\0\0\0\0\0\x7a\x07\x58\0\0\0\0\0\x7a\x07\
\x50\0\0\0\0\0\x7a\x07\x48\0\0\0\0\0\x7a\x07\x40\0\0\0\0\0\x7a\x07\x38\0\0\0\0\
\0\x7a\x07\x30\0\0\0\0\0\x7a\x07\x28\0\0\0\0\0\x7a\x07\x20\0\0\0\0\0\x7a\x07\
\x18\0\0\0\0\0\x7a\x07\x10\0\0\0\0\0\x7a\x07\x08\0\0\0\0\0\x7a\x07\0\0\0\0\0\0\
\xbf\xa1\0\0\0\0\0\0\x07\x01\0\0\xf8\xff\xff\xff\x7b\x17\x10\0\0\0\0\0\xbf\xa1\
\0\0\0\0\0\0\x07\x01\0\0\xfc\xff\xff\xff\x7b\x17\x08\0\0\0\0\0\x63\x67\0\0\0\0\
\0\0\xb4\x01\0\0\x02\0\0\0\xbf\x72\0\0\0\0\0\0\xb4\x03\0\0\x90\0\0\0\x85\0\0\0\
\xa6\0\0\0\x56\0\x1c\0\0\0\0\0\x7a\x07\x88\0\0\0\0\0\x7a\x07\x80\0\0\0\0\0\x7a\
\x07\x78\0\0\0\0\0\x7a\x07\x70\0\0\0\0\0\x7a\x07\x68\0\0\0\0\0\x7a\x07\x60\0\0\
\0\0\0\x7a\x07\x58\0\0\0\0\0\x7a\x07\x50\0\0\0\0\0\x7a\x07\x48\0\0\0\0\0\x7a\
\x07\x40\0\0\0\0\0\x7a\x07\x38\0\0\0\0\0\x7a\x07\x30\0\0\0\0\0\x7a\x07\x28\0\0\
\0\0\0\x7a\x07\x20\0\0\0\0\0\x7a\x07\x18\0\0\0\0\0\x7a\x07\x10\0\0\0\0\0\x7a\
\x07\x08\0\0\0\0\0\x7a\x07\0\0\0\0\0\0\xbf\xa1\0\0\0\0\0\0\x07\x01\0\0\xfc\xff\
\xff\xff\x7b\x17\x08\0\0\0\0\0\x63\x67\0\0\0\0\0\0\xb4\x01\0\0\x03\0\0\0\xbf\
\x72\0\0\0\0\0\0\xb4\x03\0\0\x90\0\0\0\x85\0\0\0\xa6\0\0\0\x56\0\x01\0\0\0\0\0\
\xb4\x08\0\0\x01\0\0\0\x61\xa1\xf8\xff\0\0\0\0\xc6\x01\x01\0\0\0\0\0\x85\0\0\0\
\xa8\0\0\0\xc6\x06\x02\0\0\0\0\0\xbc\x61\0\0\0\0\0\0\x85\0\0\0\xa8\0\0\0\xbc\
\x80\0\0\0\0\0\0\x95\0\0\0\0\0\0\0\x47\x50\x4c\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x47\x50\x4c\0\x7a\x0a\xf8\xff\0\0\0\0\xbf\xa2\0\
\0\0\0\0\0\x07\x02\0\0\xf8\xff\xff\xff\x18\x11\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x85\
\0\0\0\x01\0\0\0\xb7\0\0\0\0\0\0\0\x95\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\x08\0\0\
\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\x01\0\0\0\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\x0c\0\0\0\0\0\0\0\x22\0\0\0\0\0\0\0\x06\0\0\0\x08\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x3c\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\x9f\xeb\x01\0\x18\0\0\0\0\0\0\0\x20\0\0\0\x20\0\0\0\x04\0\0\
\0\0\0\0\0\0\0\0\x01\x08\0\0\0\x40\0\0\x01\0\0\0\0\0\0\0\x01\x08\0\0\0\x40\0\0\
\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\x01\0\0\0\x10\0\0\0\xd8\0\0\0\0\0\0\0\x01\
\0\0\0\x10\0\0\0\x10\x01\0\0\0\0\0\0\x01\0\0\0\x10\0\0\0\x30\x01\0\0\0\0\0\0\
\x01\0\0\0\x10\0\0\0\x40\x01\0\0\0\0\0\0\x01\0\0\0\x10\0\0\0\x58\x01\0\0\0\0\0\
\0\x01\0\0\0\x10\0\0\0\xa0\x01\0\0\0\0\0\0\x01\0\0\0\x10\0\0\0\xd0\x01\0\0\0\0\
\0\0\x01\0\0\0\x10\0\0\0\xe0\x01\0\0\0\0\0\0\x01\0\0\0\x10\0\0\0\xf8\x01\0\0\0\
\0\0\0\x01\0\0\0\x10\0\0\0\x28\x02\0\0\0\0\0\0\x01\0\0\0\x10\0\0\0\x58\x02\0\0\
\0\0\0\0\x01\0\0\0\x10\0\0\0\xb8\x02\0\0\0\0\0\0\x01\0\0\0\x13\0\0\0\x80\x03\0\
\0\0\0\0\0\x01\0\0\0\x14\0\0\0\x40\0\0\0\0\0\0\0\x02\0\0\0\x16\0\0\0\x9f\xeb\
\x01\0\x18\0\0\0\0\0\0\0\x1c\x10\0\0\x1c\x10\0\0\xe0\x0e\0\0\0\0\0\0\0\0\0\x02\
\x03\0\0\0\x01\0\0\0\0\0\0\x01\x04\0\0\0\x20\0\0\x01\0\0\0\0\0\0\0\x03\0\0\0\0\
\x02\0\0\0\x04\0\0\0\x02\0\0\0\x05\0\0\0\0\0\0\x01\x04\0\0\0\x20\0\0\0\0\0\0\0\
\0\0\0\x02\x06\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x04\0\0\0\0\
\0\0\0\0\0\0\x02\x08\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x01\0\
\0\0\x19\0\0\0\x04\0\0\x04\x20\0\0\0\x28\0\0\0\x01\0\0\0\0\0\0\0\x2d\0\0\0\x05\
\0\0\0\x40\0\0\0\x36\0\0\0\x05\0\0\0\x80\0\0\0\x41\0\0\0\x07\0\0\0\xc0\0\0\0\
\x4d\0\0\0\0\0\0\x0e\x09\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x0c\0\0\0\0\0\0\0\0\
\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x0c\0\0\0\0\0\0\0\0\0\0\x02\x02\0\0\0\0\0\
\0\0\0\0\0\x02\x09\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\x0e\0\0\0\x04\0\0\0\0\0\0\0\
\0\0\0\0\x05\0\0\x04\x20\0\0\0\x28\0\0\0\x0b\0\0\0\0\0\0\0\x57\0\0\0\x0d\0\0\0\
\x40\0\0\0\x5b\0\0\0\x0d\0\0\0\x80\0\0\0\x41\0\0\0\x07\0\0\0\xc0\0\0\0\x61\0\0\
\0\x0f\0\0\0\0\x01\0\0\x68\0\0\0\0\0\0\x0e\x10\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\
\x02\x13\0\0\0\x78\0\0\0\x13\0\0\x05\x90\0\0\0\0\0\0\0\x14\0\0\0\0\0\0\0\0\0\0\
\0\x1b\0\0\0\0\0\0\0\x81\0\0\0\x1d\0\0\0\0\0\0\0\0\0\0\0\x1e\0\0\0\0\0\0\0\0\0\
\0\0\x20\0\0\0\0\0\0\0\0\0\0\0\x22\0\0\0\0\0\0\0\x87\0\0\0\x25\0\0\0\0\0\0\0\0\
\0\0\0\x26\0\0\0\0\0\0\0\x8c\0\0\0\x28\0\0\0\0\0\0\0\x91\0\0\0\x29\0\0\0\0\0\0\
\0\x97\0\0\0\x2b\0\0\0\0\0\0\0\0\0\0\0\x2c\0\0\0\0\0\0\0\xa6\0\0\0\x2d\0\0\0\0\
\0\0\0\xb4\0\0\0\x2e\0\0\0\0\0\0\0\xc0\0\0\0\x38\0\0\0\0\0\0\0\xcc\0\0\0\x3b\0\
\0\0\0\0\0\0\xd8\0\0\0\x3c\0\0\0\0\0\0\0\xe5\0\0\0\x3d\0\0\0\0\0\0\0\xf1\0\0\0\
\x3e\0\0\0\0\0\0\0\0\0\0\0\x0e\0\0\x04\x48\0\0\0\xff\0\0\0\x15\0\0\0\0\0\0\0\
\x2d\0\0\0\x15\0\0\0\x20\0\0\0\x36\0\0\0\x15\0\0\0\x40\0\0\0\x41\0\0\0\x15\0\0\
\0\x60\0\0\0\x08\x01\0\0\x15\0\0\0\x80\0\0\0\x12\x01\0\0\x15\0\0\0\xa0\0\0\0\
\x1f\x01\0\0\x15\0\0\0\xc0\0\0\0\x29\x01\0\0\x18\0\0\0\xe0\0\0\0\x32\x01\0\0\
\x15\0\0\0\x60\x01\0\0\x3e\x01\0\0\x15\0\0\0\x80\x01\0\0\x45\x01\0\0\x15\0\0\0\
\xa0\x01\0\0\x55\x01\0\0\x15\0\0\0\xc0\x01\0\0\x67\x01\0\0\x15\0\0\0\xe0\x01\0\
\0\x81\x01\0\0\x19\0\0\0\0\x02\0\0\x8b\x01\0\0\0\0\0\x08\x16\0\0\0\x91\x01\0\0\
\0\0\0\x01\x04\0\0\0\x20\0\0\0\x9e\x01\0\0\0\0\0\x01\x01\0\0\0\x08\0\0\x01\0\0\
\0\0\0\0\0\x03\0\0\0\0\x17\0\0\0\x04\0\0\0\x10\0\0\0\xa3\x01\0\0\0\0\0\x08\x1a\
\0\0\0\xa9\x01\0\0\0\0\0\x01\x08\0\0\0\x40\0\0\0\0\0\0\0\x04\0\0\x04\x20\0\0\0\
\xbc\x01\0\0\x15\0\0\0\0\0\0\0\x57\0\0\0\x19\0\0\0\x40\0\0\0\0\0\0\0\x1c\0\0\0\
\x80\0\0\0\xc3\x01\0\0\x19\0\0\0\xc0\0\0\0\0\0\0\0\x02\0\0\x05\x08\0\0\0\x5b\0\
\0\0\x19\0\0\0\0\0\0\0\xc9\x01\0\0\x19\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\x04\x38\0\
\0\0\xd2\x01\0\0\x19\0\0\0\0\0\0\0\xdb\x01\0\0\x19\0\0\0\x40\0\0\0\xe5\x01\0\0\
\x19\0\0\0\x80\0\0\0\x61\0\0\0\x19\0\0\0\xc0\0\0\0\xea\x01\0\0\x15\0\0\0\0\x01\
\0\0\xbc\x01\0\0\x15\0\0\0\x20\x01\0\0\xf0\x01\0\0\x19\0\0\0\x40\x01\0\0\xc3\
\x01\0\0\x19\0\0\0\x80\x01\0\0\0\0\0\0\x1a\0\0\x04\x90\0\0\0\xfb\x01\0\0\x15\0\
\0\0\0\0\0\0\x05\x02\0\0\x15\0\0\0\x20\0\0\0\x0e\x02\0\0\x19\0\0\0\x40\0\0\0\
\x14\x02\0\0\x19\0\0\0\x80\0\0\0\x1c\x02\0\0\x15\0\0\0\xc0\0\0\0\x26\x02\0\0\
\x15\0\0\0\xe0\0\0\0\x2f\x02\0\0\x19\0\0\0\0\x01\0\0\x37\x02\0\0\x15\0\0\0\x40\
\x01\0\0\x44\x02\0\0\x15\0\0\0\x60\x01\0\0\x4f\x02\0\0\x18\0\0\0\x80\x01\0\0\
\x59\x02\0\0\x15\0\0\0\0\x02\0\0\x66\x02\0\0\x15\0\0\0\x20\x02\0\0\x7b\x02\0\0\
\x15\0\0\0\x40\x02\0\0\x87\x02\0\0\x15\0\0\0\x60\x02\0\0\x9a\x02\0\0\x19\0\0\0\
\x80\x02\0\0\xa4\x02\0\0\x15\0\0\0\xc0\x02\0\0\xb2\x02\0\0\x15\0\0\0\xe0\x02\0\
\0\xc5\x02\0\0\x19\0\0\0\0\x03\0\0\xcf\x02\0\0\x15\0\0\0\x40\x03\0\0\xdd\x02\0\
\0\x15\0\0\0\x60\x03\0\0\0\0\0\0\x1f\0\0\0\x80\x03\0\0\xeb\x02\0\0\x15\0\0\0\
\xa0\x03\0\0\xf9\x02\0\0\x19\0\0\0\xc0\x03\0\0\x02\x03\0\0\x19\0\0\0\0\x04\0\0\
\x0d\x03\0\0\x15\0\0\0\x40\x04\0\0\x20\x03\0\0\x15\0\0\0\x60\x04\0\0\0\0\0\0\
\x02\0\0\x05\x04\0\0\0\x2e\x03\0\0\x15\0\0\0\0\0\0\0\x3d\x03\0\0\x15\0\0\0\0\0\
\0\0\0\0\0\0\x04\0\0\x04\x18\0\0\0\x4f\x03\0\0\x19\0\0\0\0\0\0\0\x58\x03\0\0\
\x15\0\0\0\x40\0\0\0\x5f\x03\0\0\x15\0\0\0\x60\0\0\0\x6a\x03\0\0\x21\0\0\0\x80\
\0\0\0\x72\x03\0\0\0\0\0\x08\x02\0\0\0\0\0\0\0\x07\0\0\x04\x20\0\0\0\0\0\0\0\
\x23\0\0\0\0\0\0\0\x78\x03\0\0\x15\0\0\0\x20\0\0\0\x86\x03\0\0\x15\0\0\0\x40\0\
\0\0\x92\x03\0\0\x15\0\0\0\x60\0\0\0\x9f\x03\0\0\x15\0\0\0\x80\0\0\0\0\0\0\0\
\x24\0\0\0\xa0\0\0\0\xae\x03\0\0\x19\0\0\0\xc0\0\0\0\0\0\0\0\x02\0\0\x05\x04\0\
\0\0\xc0\x03\0\0\x15\0\0\0\0\0\0\0\xca\x03\0\0\x15\0\0\0\0\0\0\0\0\0\0\0\x02\0\
\0\x05\x04\0\0\0\xd9\x03\0\0\x15\0\0\0\0\0\0\0\xe5\x03\0\0\x15\0\0\0\0\0\0\0\0\
\0\0\0\x0f\0\0\x04\x50\0\0\0\xf1\x03\0\0\x15\0\0\0\0\0\0\0\xf9\x03\0\0\x15\0\0\
\0\x20\0\0\0\0\x04\0\0\x15\0\0\0\x40\0\0\0\x0d\x04\0\0\x15\0\0\0\x60\0\0\0\x1b\
\x04\0\0\x19\0\0\0\x80\0\0\0\x23\x04\0\0\x19\0\0\0\xc0\0\0\0\x2c\x04\0\0\x15\0\
\0\0\0\x01\0\0\x33\x04\0\0\x15\0\0\0\x20\x01\0\0\x3c\x04\0\0\x15\0\0\0\x40\x01\
\0\0\x48\x04\0\0\x15\0\0\0\x60\x01\0\0\x55\x04\0\0\x19\0\0\0\x80\x01\0\0\x5c\
\x04\0\0\x19\0\0\0\xc0\x01\0\0\xc3\x01\0\0\x15\0\0\0\0\x02\0\0\x64\x04\0\0\x15\
\0\0\0\x20\x02\0\0\x68\x04\0\0\x15\0\0\0\x40\x02\0\0\0\0\0\0\x03\0\0\x04\x0c\0\
\0\0\0\0\0\0\x27\0\0\0\0\0\0\0\x73\x04\0\0\x15\0\0\0\x20\0\0\0\x7b\x04\0\0\x15\
\0\0\0\x40\0\0\0\0\0\0\0\x05\0\0\x05\x04\0\0\0\x86\x04\0\0\x15\0\0\0\0\0\0\0\
\x8f\x04\0\0\x15\0\0\0\0\0\0\0\x97\x04\0\0\x15\0\0\0\0\0\0\0\x9e\x04\0\0\x15\0\
\0\0\0\0\0\0\xa5\x04\0\0\x15\0\0\0\0\0\0\0\0\0\0\0\x03\0\0\x04\x10\0\0\0\x58\
\x03\0\0\x15\0\0\0\0\0\0\0\xad\x04\0\0\x15\0\0\0\x20\0\0\0\x8c\0\0\0\x19\0\0\0\
\x40\0\0\0\0\0\0\0\x0a\0\0\x04\x40\0\0\0\0\0\0\0\x23\0\0\0\0\0\0\0\x86\x03\0\0\
\x15\0\0\0\x20\0\0\0\xb6\x04\0\0\x15\0\0\0\x40\0\0\0\x92\x03\0\0\x15\0\0\0\x60\
\0\0\0\xc2\x04\0\0\x19\0\0\0\x80\0\0\0\0\0\0\0\x2a\0\0\0\xc0\0\0\0\xcb\x04\0\0\
\x19\0\0\0\0\x01\0\0\xdd\x04\0\0\x19\0\0\0\x40\x01\0\0\xe6\x04\0\0\x19\0\0\0\
\x80\x01\0\0\xf8\x04\0\0\x19\0\0\0\xc0\x01\0\0\0\0\0\0\x02\0\0\x05\x04\0\0\0\
\x01\x05\0\0\x15\0\0\0\0\0\0\0\xea\x01\0\0\x15\0\0\0\0\0\0\0\0\0\0\0\x02\0\0\
\x04\x10\0\0\0\x0a\x05\0\0\x19\0\0\0\0\0\0\0\xf1\x03\0\0\x15\0\0\0\x40\0\0\0\0\
\0\0\0\x06\0\0\x04\x20\0\0\0\x0f\x05\0\0\x19\0\0\0\0\0\0\0\x13\x05\0\0\x19\0\0\
\0\x40\0\0\0\x1f\x05\0\0\x15\0\0\0\x80\0\0\0\x28\x05\0\0\x15\0\0\0\xa0\0\0\0\
\x35\x05\0\0\x15\0\0\0\xc0\0\0\0\x43\x05\0\0\x15\0\0\0\xe0\0\0\0\0\0\0\0\x09\0\
\0\x04\x30\0\0\0\x55\x05\0\0\x15\0\0\0\0\0\0\0\x59\x05\0\0\x15\0\0\0\x20\0\0\0\
\xc3\x01\0\0\x15\0\0\0\x40\0\0\0\x5c\x05\0\0\x15\0\0\0\x60\0\0\0\x64\x05\0\0\
\x19\0\0\0\x80\0\0\0\x8f\x04\0\0\x15\0\0\0\xc0\0\0\0\x68\x05\0\0\x15\0\0\0\xe0\
\0\0\0\x70\x05\0\0\x19\0\0\0\0\x01\0\0\x7d\x05\0\0\x19\0\0\0\x40\x01\0\0\0\0\0\
\0\x05\0\0\x04\x40\0\0\0\0\0\0\0\x2f\0\0\0\0\0\0\0\0\0\0\0\x23\0\0\0\x20\0\0\0\
\x86\x03\0\0\x15\0\0\0\x40\0\0\0\xc3\x01\0\0\x15\0\0\0\x60\0\0\0\0\0\0\0\x30\0\
\0\0\x80\0\0\0\0\0\0\0\x02\0\0\x05\x04\0\0\0\xf1\x03\0\0\x15\0\0\0\0\0\0\0\xbc\
\x01\0\0\x15\0\0\0\0\0\0\0\0\0\0\0\x09\0\0\x05\x30\0\0\0\x88\x05\0\0\x15\0\0\0\
\0\0\0\0\0\0\0\0\x31\0\0\0\0\0\0\0\x96\x05\0\0\x32\0\0\0\0\0\0\0\xa1\x05\0\0\
\x33\0\0\0\0\0\0\0\xae\x05\0\0\x34\0\0\0\0\0\0\0\xb6\x05\0\0\x35\0\0\0\0\0\0\0\
\xc0\x05\0\0\x36\0\0\0\0\0\0\0\xc4\x05\0\0\x37\0\0\0\0\0\0\0\xd1\x05\0\0\x36\0\
\0\0\0\0\0\0\0\0\0\0\x02\0\0\x04\x10\0\0\0\xd8\x05\0\0\x19\0\0\0\0\0\0\0\xe2\
\x05\0\0\x15\0\0\0\x40\0\0\0\0\0\0\0\x01\0\0\x04\x08\0\0\0\xf0\x05\0\0\x19\0\0\
\0\0\0\0\0\0\0\0\0\x05\0\0\x04\x20\0\0\0\xc3\x01\0\0\x15\0\0\0\0\0\0\0\xfb\x05\
\0\0\x15\0\0\0\x20\0\0\0\xff\x05\0\0\x19\0\0\0\x40\0\0\0\x04\x06\0\0\x19\0\0\0\
\x80\0\0\0\x0a\x06\0\0\x19\0\0\0\xc0\0\0\0\0\0\0\0\x02\0\0\x04\x10\0\0\0\x88\
\x05\0\0\x15\0\0\0\0\0\0\0\x12\x06\0\0\x19\0\0\0\x40\0\0\0\0\0\0\0\x04\0\0\x04\
\x10\0\0\0\x19\x06\0\0\x15\0\0\0\0\0\0\0\x1c\x06\0\0\x15\0\0\0\x20\0\0\0\x24\
\x06\0\0\x21\0\0\0\x40\0\0\0\xc3\x01\0\0\x15\0\0\0\x60\0\0\0\0\0\0\0\x02\0\0\
\x04\x10\0\0\0\0\0\0\0\x24\0\0\0\0\0\0\0\xae\x03\0\0\x19\0\0\0\x40\0\0\0\0\0\0\
\0\x07\0\0\x04\x30\0\0\0\x2d\x06\0\0\x19\0\0\0\0\0\0\0\x32\x06\0\0\x19\0\0\0\
\x40\0\0\0\x3a\x06\0\0\x19\0\0\0\x80\0\0\0\x0a\x06\0\0\x19\0\0\0\xc0\0\0\0\xfb\
\x05\0\0\x15\0\0\0\0\x01\0\0\xc3\x01\0\0\x15\0\0\0\x20\x01\0\0\x55\x05\0\0\x15\
\0\0\0\x40\x01\0\0\0\0\0\0\x04\0\0\x04\x10\0\0\0\x4a\x06\0\0\x15\0\0\0\0\0\0\0\
\0\0\0\0\x39\0\0\0\x20\0\0\0\xc3\x01\0\0\x15\0\0\0\x40\0\0\0\0\0\0\0\x3a\0\0\0\
\x60\0\0\0\0\0\0\0\x02\0\0\x05\x04\0\0\0\x52\x06\0\0\x15\0\0\0\0\0\0\0\x5e\x06\
\0\0\x15\0\0\0\0\0\0\0\0\0\0\0\x02\0\0\x05\x04\0\0\0\x69\x06\0\0\x15\0\0\0\0\0\
\0\0\x75\x06\0\0\x15\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\x04\x04\0\0\0\x4a\x06\0\0\
\x15\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\x04\x04\0\0\0\x28\0\0\0\x15\0\0\0\0\0\0\0\0\
\0\0\0\x02\0\0\x04\x08\0\0\0\x4a\x06\0\0\x15\0\0\0\0\0\0\0\xc3\x01\0\0\x15\0\0\
\0\x20\0\0\0\0\0\0\0\x03\0\0\x04\x0c\0\0\0\xf1\x03\0\0\x15\0\0\0\0\0\0\0\xbc\
\x01\0\0\x15\0\0\0\x20\0\0\0\xc3\x01\0\0\x15\0\0\0\x40\0\0\0\0\0\0\0\x04\0\0\
\x04\x20\0\0\0\x28\0\0\0\x01\0\0\0\0\0\0\0\x57\0\0\0\x0d\0\0\0\x40\0\0\0\x5b\0\
\0\0\x12\0\0\0\x80\0\0\0\x41\0\0\0\x07\0\0\0\xc0\0\0\0\x80\x06\0\0\0\0\0\x0e\
\x3f\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x42\0\0\0\x8f\x06\0\0\x06\0\0\x04\x20\0\
\0\0\x2f\x02\0\0\x19\0\0\0\0\0\0\0\x26\x02\0\0\x15\0\0\0\x40\0\0\0\x41\0\0\0\
\x02\0\0\0\x60\0\0\0\xbc\x01\0\0\x02\0\0\0\x80\0\0\0\xf1\x03\0\0\x02\0\0\0\xa0\
\0\0\0\x3e\x01\0\0\x02\0\0\0\xc0\0\0\0\0\0\0\0\x01\0\0\x0d\x02\0\0\0\x94\x06\0\
\0\x41\0\0\0\x98\x06\0\0\x01\0\0\x0c\x43\0\0\0\0\0\0\0\0\0\0\x02\0\0\0\0\0\0\0\
\0\x01\0\0\x0d\x02\0\0\0\x94\x06\0\0\x45\0\0\0\xa2\x06\0\0\x01\0\0\x0c\x46\0\0\
\0\xb3\x06\0\0\x01\0\0\x04\x04\0\0\0\xbb\x06\0\0\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\x03\0\0\0\0\x17\0\0\0\x04\0\0\0\x04\0\0\0\xbe\x06\0\0\0\0\0\x0e\x49\0\0\0\
\x01\0\0\0\xc7\x06\0\0\0\0\0\x0e\x49\0\0\0\0\0\0\0\xd9\x06\0\0\x05\0\0\x84\x08\
\0\0\0\xe2\x06\0\0\x4d\0\0\0\0\0\0\0\xe7\x06\0\0\x4d\0\0\0\x08\0\0\x04\xef\x06\
\0\0\x4d\0\0\0\x0c\0\0\x04\xf7\x06\0\0\x4f\0\0\0\x10\0\0\0\xfb\x06\0\0\x21\0\0\
\0\x20\0\0\0\xff\x06\0\0\0\0\0\x08\x4e\0\0\0\x04\x07\0\0\0\0\0\x01\x01\0\0\0\
\x08\0\0\0\x12\x07\0\0\0\0\0\x08\x50\0\0\0\x18\x07\0\0\0\0\0\x01\x02\0\0\0\x10\
\0\0\x01\0\0\0\0\0\0\0\x03\0\0\0\0\x4c\0\0\0\x04\0\0\0\x08\0\0\0\x1e\x07\0\0\0\
\0\0\x0e\x51\0\0\0\0\0\0\0\x2e\x07\0\0\0\0\0\x0e\x13\0\0\0\0\0\0\0\x48\x07\0\0\
\0\0\0\x0e\x13\0\0\0\0\0\0\0\x62\x07\0\0\0\0\0\x0e\x19\0\0\0\0\0\0\0\x70\x07\0\
\0\0\0\0\x0e\x19\0\0\0\0\0\0\0\x80\x07\0\0\0\0\0\x0e\x13\0\0\0\0\0\0\0\x99\x07\
\0\0\0\0\0\x0e\x13\0\0\0\0\0\0\0\x14\x02\0\0\x01\0\0\x0f\x04\0\0\0\x4a\0\0\0\0\
\0\0\0\x04\0\0\0\xc4\x0e\0\0\x03\0\0\x0f\x68\0\0\0\x0a\0\0\0\0\0\0\0\x20\0\0\0\
\x11\0\0\0\x20\0\0\0\x28\0\0\0\x40\0\0\0\x48\0\0\0\x20\0\0\0\xca\x0e\0\0\x08\0\
\0\x0f\x98\x02\0\0\x4b\0\0\0\0\0\0\0\x04\0\0\0\x52\0\0\0\x04\0\0\0\x40\0\0\0\
\x53\0\0\0\x48\0\0\0\x90\0\0\0\x54\0\0\0\xd8\0\0\0\x90\0\0\0\x55\0\0\0\x68\x01\
\0\0\x08\0\0\0\x56\0\0\0\x70\x01\0\0\x08\0\0\0\x57\0\0\0\x78\x01\0\0\x90\0\0\0\
\x58\0\0\0\x08\x02\0\0\x90\0\0\0\xd0\x0e\0\0\0\0\0\x0f\x3c\0\0\0\0\x69\x6e\x74\
\0\x5f\x5f\x41\x52\x52\x41\x59\x5f\x53\x49\x5a\x45\x5f\x54\x59\x50\x45\x5f\x5f\
\0\x69\x6e\x6e\x65\x72\x5f\x6d\x61\x70\x5f\x74\x79\x70\x65\0\x74\x79\x70\x65\0\
\x6b\x65\x79\x5f\x73\x69\x7a\x65\0\x76\x61\x6c\x75\x65\x5f\x73\x69\x7a\x65\0\
\x6d\x61\x78\x5f\x65\x6e\x74\x72\x69\x65\x73\0\x69\x6e\x6e\x65\x72\x5f\x6d\x61\
\x70\0\x6b\x65\x79\0\x76\x61\x6c\x75\x65\0\x76\x61\x6c\x75\x65\x73\0\x6f\x75\
\x74\x65\x72\x5f\x61\x72\x72\x61\x79\x5f\x6d\x61\x70\0\x62\x70\x66\x5f\x61\x74\
\x74\x72\0\x62\x61\x74\x63\x68\0\x74\x65\x73\x74\0\x69\x6e\x66\x6f\0\x71\x75\
\x65\x72\x79\0\x72\x61\x77\x5f\x74\x72\x61\x63\x65\x70\x6f\x69\x6e\x74\0\x74\
\x61\x73\x6b\x5f\x66\x64\x5f\x71\x75\x65\x72\x79\0\x6c\x69\x6e\x6b\x5f\x63\x72\
\x65\x61\x74\x65\0\x6c\x69\x6e\x6b\x5f\x75\x70\x64\x61\x74\x65\0\x6c\x69\x6e\
\x6b\x5f\x64\x65\x74\x61\x63\x68\0\x65\x6e\x61\x62\x6c\x65\x5f\x73\x74\x61\x74\
\x73\0\x69\x74\x65\x72\x5f\x63\x72\x65\x61\x74\x65\0\x70\x72\x6f\x67\x5f\x62\
\x69\x6e\x64\x5f\x6d\x61\x70\0\x6d\x61\x70\x5f\x74\x79\x70\x65\0\x6d\x61\x70\
\x5f\x66\x6c\x61\x67\x73\0\x69\x6e\x6e\x65\x72\x5f\x6d\x61\x70\x5f\x66\x64\0\
\x6e\x75\x6d\x61\x5f\x6e\x6f\x64\x65\0\x6d\x61\x70\x5f\x6e\x61\x6d\x65\0\x6d\
\x61\x70\x5f\x69\x66\x69\x6e\x64\x65\x78\0\x62\x74\x66\x5f\x66\x64\0\x62\x74\
\x66\x5f\x6b\x65\x79\x5f\x74\x79\x70\x65\x5f\x69\x64\0\x62\x74\x66\x5f\x76\x61\
\x6c\x75\x65\x5f\x74\x79\x70\x65\x5f\x69\x64\0\x62\x74\x66\x5f\x76\x6d\x6c\x69\
\x6e\x75\x78\x5f\x76\x61\x6c\x75\x65\x5f\x74\x79\x70\x65\x5f\x69\x64\0\x6d\x61\
\x70\x5f\x65\x78\x74\x72\x61\0\x5f\x5f\x75\x33\x32\0\x75\x6e\x73\x69\x67\x6e\
\x65\x64\x20\x69\x6e\x74\0\x63\x68\x61\x72\0\x5f\x5f\x75\x36\x34\0\x75\x6e\x73\
\x69\x67\x6e\x65\x64\x20\x6c\x6f\x6e\x67\x20\x6c\x6f\x6e\x67\0\x6d\x61\x70\x5f\
\x66\x64\0\x66\x6c\x61\x67\x73\0\x6e\x65\x78\x74\x5f\x6b\x65\x79\0\x69\x6e\x5f\
\x62\x61\x74\x63\x68\0\x6f\x75\x74\x5f\x62\x61\x74\x63\x68\0\x6b\x65\x79\x73\0\
\x63\x6f\x75\x6e\x74\0\x65\x6c\x65\x6d\x5f\x66\x6c\x61\x67\x73\0\x70\x72\x6f\
\x67\x5f\x74\x79\x70\x65\0\x69\x6e\x73\x6e\x5f\x63\x6e\x74\0\x69\x6e\x73\x6e\
\x73\0\x6c\x69\x63\x65\x6e\x73\x65\0\x6c\x6f\x67\x5f\x6c\x65\x76\x65\x6c\0\x6c\
\x6f\x67\x5f\x73\x69\x7a\x65\0\x6c\x6f\x67\x5f\x62\x75\x66\0\x6b\x65\x72\x6e\
\x5f\x76\x65\x72\x73\x69\x6f\x6e\0\x70\x72\x6f\x67\x5f\x66\x6c\x61\x67\x73\0\
\x70\x72\x6f\x67\x5f\x6e\x61\x6d\x65\0\x70\x72\x6f\x67\x5f\x69\x66\x69\x6e\x64\
\x65\x78\0\x65\x78\x70\x65\x63\x74\x65\x64\x5f\x61\x74\x74\x61\x63\x68\x5f\x74\
\x79\x70\x65\0\x70\x72\x6f\x67\x5f\x62\x74\x66\x5f\x66\x64\0\x66\x75\x6e\x63\
\x5f\x69\x6e\x66\x6f\x5f\x72\x65\x63\x5f\x73\x69\x7a\x65\0\x66\x75\x6e\x63\x5f\
\x69\x6e\x66\x6f\0\x66\x75\x6e\x63\x5f\x69\x6e\x66\x6f\x5f\x63\x6e\x74\0\x6c\
\x69\x6e\x65\x5f\x69\x6e\x66\x6f\x5f\x72\x65\x63\x5f\x73\x69\x7a\x65\0\x6c\x69\
\x6e\x65\x5f\x69\x6e\x66\x6f\0\x6c\x69\x6e\x65\x5f\x69\x6e\x66\x6f\x5f\x63\x6e\
\x74\0\x61\x74\x74\x61\x63\x68\x5f\x62\x74\x66\x5f\x69\x64\0\x63\x6f\x72\x65\
\x5f\x72\x65\x6c\x6f\x5f\x63\x6e\x74\0\x66\x64\x5f\x61\x72\x72\x61\x79\0\x63\
\x6f\x72\x65\x5f\x72\x65\x6c\x6f\x73\0\x63\x6f\x72\x65\x5f\x72\x65\x6c\x6f\x5f\
\x72\x65\x63\x5f\x73\x69\x7a\x65\0\x6c\x6f\x67\x5f\x74\x72\x75\x65\x5f\x73\x69\
\x7a\x65\0\x61\x74\x74\x61\x63\x68\x5f\x70\x72\x6f\x67\x5f\x66\x64\0\x61\x74\
\x74\x61\x63\x68\x5f\x62\x74\x66\x5f\x6f\x62\x6a\x5f\x66\x64\0\x70\x61\x74\x68\
\x6e\x61\x6d\x65\0\x62\x70\x66\x5f\x66\x64\0\x66\x69\x6c\x65\x5f\x66\x6c\x61\
\x67\x73\0\x70\x61\x74\x68\x5f\x66\x64\0\x5f\x5f\x73\x33\x32\0\x61\x74\x74\x61\
\x63\x68\x5f\x62\x70\x66\x5f\x66\x64\0\x61\x74\x74\x61\x63\x68\x5f\x74\x79\x70\
\x65\0\x61\x74\x74\x61\x63\x68\x5f\x66\x6c\x61\x67\x73\0\x72\x65\x70\x6c\x61\
\x63\x65\x5f\x62\x70\x66\x5f\x66\x64\0\x65\x78\x70\x65\x63\x74\x65\x64\x5f\x72\
\x65\x76\x69\x73\x69\x6f\x6e\0\x74\x61\x72\x67\x65\x74\x5f\x66\x64\0\x74\x61\
\x72\x67\x65\x74\x5f\x69\x66\x69\x6e\x64\x65\x78\0\x72\x65\x6c\x61\x74\x69\x76\
\x65\x5f\x66\x64\0\x72\x65\x6c\x61\x74\x69\x76\x65\x5f\x69\x64\0\x70\x72\x6f\
\x67\x5f\x66\x64\0\x72\x65\x74\x76\x61\x6c\0\x64\x61\x74\x61\x5f\x73\x69\x7a\
\x65\x5f\x69\x6e\0\x64\x61\x74\x61\x5f\x73\x69\x7a\x65\x5f\x6f\x75\x74\0\x64\
\x61\x74\x61\x5f\x69\x6e\0\x64\x61\x74\x61\x5f\x6f\x75\x74\0\x72\x65\x70\x65\
\x61\x74\0\x64\x75\x72\x61\x74\x69\x6f\x6e\0\x63\x74\x78\x5f\x73\x69\x7a\x65\
\x5f\x69\x6e\0\x63\x74\x78\x5f\x73\x69\x7a\x65\x5f\x6f\x75\x74\0\x63\x74\x78\
\x5f\x69\x6e\0\x63\x74\x78\x5f\x6f\x75\x74\0\x63\x70\x75\0\x62\x61\x74\x63\x68\
\x5f\x73\x69\x7a\x65\0\x6e\x65\x78\x74\x5f\x69\x64\0\x6f\x70\x65\x6e\x5f\x66\
\x6c\x61\x67\x73\0\x73\x74\x61\x72\x74\x5f\x69\x64\0\x70\x72\x6f\x67\x5f\x69\
\x64\0\x6d\x61\x70\x5f\x69\x64\0\x62\x74\x66\x5f\x69\x64\0\x6c\x69\x6e\x6b\x5f\
\x69\x64\0\x69\x6e\x66\x6f\x5f\x6c\x65\x6e\0\x71\x75\x65\x72\x79\x5f\x66\x6c\
\x61\x67\x73\0\x70\x72\x6f\x67\x5f\x69\x64\x73\0\x70\x72\x6f\x67\x5f\x61\x74\
\x74\x61\x63\x68\x5f\x66\x6c\x61\x67\x73\0\x6c\x69\x6e\x6b\x5f\x69\x64\x73\0\
\x6c\x69\x6e\x6b\x5f\x61\x74\x74\x61\x63\x68\x5f\x66\x6c\x61\x67\x73\0\x72\x65\
\x76\x69\x73\x69\x6f\x6e\0\x70\x72\x6f\x67\x5f\x63\x6e\x74\0\x6e\x61\x6d\x65\0\
\x62\x74\x66\0\x62\x74\x66\x5f\x6c\x6f\x67\x5f\x62\x75\x66\0\x62\x74\x66\x5f\
\x73\x69\x7a\x65\0\x62\x74\x66\x5f\x6c\x6f\x67\x5f\x73\x69\x7a\x65\0\x62\x74\
\x66\x5f\x6c\x6f\x67\x5f\x6c\x65\x76\x65\x6c\0\x62\x74\x66\x5f\x6c\x6f\x67\x5f\
\x74\x72\x75\x65\x5f\x73\x69\x7a\x65\0\x70\x69\x64\0\x66\x64\0\x62\x75\x66\x5f\
\x6c\x65\x6e\0\x62\x75\x66\0\x66\x64\x5f\x74\x79\x70\x65\0\x70\x72\x6f\x62\x65\
\x5f\x6f\x66\x66\x73\x65\x74\0\x70\x72\x6f\x62\x65\x5f\x61\x64\x64\x72\0\x74\
\x61\x72\x67\x65\x74\x5f\x62\x74\x66\x5f\x69\x64\0\x70\x65\x72\x66\x5f\x65\x76\
\x65\x6e\x74\0\x6b\x70\x72\x6f\x62\x65\x5f\x6d\x75\x6c\x74\x69\0\x74\x72\x61\
\x63\x69\x6e\x67\0\x6e\x65\x74\x66\x69\x6c\x74\x65\x72\0\x74\x63\x78\0\x75\x70\
\x72\x6f\x62\x65\x5f\x6d\x75\x6c\x74\x69\0\x6e\x65\x74\x6b\x69\x74\0\x69\x74\
\x65\x72\x5f\x69\x6e\x66\x6f\0\x69\x74\x65\x72\x5f\x69\x6e\x66\x6f\x5f\x6c\x65\
\x6e\0\x62\x70\x66\x5f\x63\x6f\x6f\x6b\x69\x65\0\x63\x6e\x74\0\x73\x79\x6d\x73\
\0\x61\x64\x64\x72\x73\0\x63\x6f\x6f\x6b\x69\x65\x73\0\x63\x6f\x6f\x6b\x69\x65\
\0\x70\x66\0\x68\x6f\x6f\x6b\x6e\x75\x6d\0\x70\x72\x69\x6f\x72\x69\x74\x79\0\
\x70\x61\x74\x68\0\x6f\x66\x66\x73\x65\x74\x73\0\x72\x65\x66\x5f\x63\x74\x72\
\x5f\x6f\x66\x66\x73\x65\x74\x73\0\x6c\x69\x6e\x6b\x5f\x66\x64\0\x6e\x65\x77\
\x5f\x70\x72\x6f\x67\x5f\x66\x64\0\x6e\x65\x77\x5f\x6d\x61\x70\x5f\x66\x64\0\
\x6f\x6c\x64\x5f\x70\x72\x6f\x67\x5f\x66\x64\0\x6f\x6c\x64\x5f\x6d\x61\x70\x5f\
\x66\x64\0\x62\x70\x66\x5f\x61\x74\x74\x72\x5f\x61\x72\x72\x61\x79\0\x61\x72\
\x67\x73\0\x63\x74\x78\0\x6c\x6f\x61\x64\x5f\x70\x72\x6f\x67\0\x75\x70\x64\x61\
\x74\x65\x5f\x6f\x75\x74\x65\x72\x5f\x6d\x61\x70\0\x62\x70\x66\x5f\x6d\x61\x70\
\0\x69\x64\0\x5f\x6c\x69\x63\x65\x6e\x73\x65\0\x6c\x6f\x61\x64\x5f\x70\x72\x6f\
\x67\x2e\x6c\x69\x63\x65\x6e\x73\x65\0\x62\x70\x66\x5f\x69\x6e\x73\x6e\0\x63\
\x6f\x64\x65\0\x64\x73\x74\x5f\x72\x65\x67\0\x73\x72\x63\x5f\x72\x65\x67\0\x6f\
\x66\x66\0\x69\x6d\x6d\0\x5f\x5f\x75\x38\0\x75\x6e\x73\x69\x67\x6e\x65\x64\x20\
\x63\x68\x61\x72\0\x5f\x5f\x73\x31\x36\0\x73\x68\x6f\x72\x74\0\x6c\x6f\x61\x64\
\x5f\x70\x72\x6f\x67\x2e\x69\x6e\x73\x6e\x73\0\x6c\x6f\x61\x64\x5f\x70\x72\x6f\
\x67\x2e\x6d\x61\x70\x5f\x63\x72\x65\x61\x74\x65\x5f\x61\x74\x74\x72\0\x6c\x6f\
\x61\x64\x5f\x70\x72\x6f\x67\x2e\x6d\x61\x70\x5f\x75\x70\x64\x61\x74\x65\x5f\
\x61\x74\x74\x72\0\x6c\x6f\x61\x64\x5f\x70\x72\x6f\x67\x2e\x6b\x65\x79\0\x6c\
\x6f\x61\x64\x5f\x70\x72\x6f\x67\x2e\x76\x61\x6c\x75\x65\0\x6c\x6f\x61\x64\x5f\
\x70\x72\x6f\x67\x2e\x70\x72\x6f\x67\x5f\x6c\x6f\x61\x64\x5f\x61\x74\x74\x72\0\
\x62\x74\x66\x5f\x6c\x6f\x61\x64\x2e\x62\x74\x66\x5f\x6c\x6f\x61\x64\x5f\x61\
\x74\x74\x72\0\x2f\x62\x75\x69\x6c\x64\x64\x69\x72\x2f\x62\x75\x69\x6c\x64\x2f\
\x42\x55\x49\x4c\x44\x2f\x6b\x65\x72\x6e\x65\x6c\x2d\x35\x2e\x31\x34\x2e\x30\
\x2d\x35\x31\x31\x2e\x72\x70\x69\x36\x34\x2e\x65\x6c\x39\x2f\x6c\x69\x6e\x75\
\x78\x2d\x35\x2e\x31\x34\x2e\x30\x2d\x35\x31\x31\x2e\x72\x70\x69\x36\x34\x2e\
\x65\x6c\x39\x2e\x61\x61\x72\x63\x68\x36\x34\x2f\x74\x6f\x6f\x6c\x73\x2f\x74\
\x65\x73\x74\x69\x6e\x67\x2f\x73\x65\x6c\x66\x74\x65\x73\x74\x73\x2f\x62\x70\
\x66\x2f\x70\x72\x6f\x67\x73\x2f\x73\x79\x73\x63\x61\x6c\x6c\x2e\x63\0\x69\x6e\
\x74\x20\x6c\x6f\x61\x64\x5f\x70\x72\x6f\x67\x28\x73\x74\x72\x75\x63\x74\x20\
\x61\x72\x67\x73\x20\x2a\x63\x74\x78\x29\0\x09\x62\x74\x66\x5f\x6c\x6f\x61\x64\
\x5f\x61\x74\x74\x72\x2e\x62\x74\x66\x20\x3d\x20\x28\x6c\x6f\x6e\x67\x29\x26\
\x72\x61\x77\x5f\x62\x74\x66\x3b\0\x09\x7d\x20\x72\x61\x77\x5f\x62\x74\x66\x20\
\x3d\x20\x7b\0\x09\x72\x65\x74\x75\x72\x6e\x20\x62\x70\x66\x5f\x73\x79\x73\x5f\
\x62\x70\x66\x28\x42\x50\x46\x5f\x42\x54\x46\x5f\x4c\x4f\x41\x44\x2c\x20\x26\
\x62\x74\x66\x5f\x6c\x6f\x61\x64\x5f\x61\x74\x74\x72\x2c\x20\x73\x69\x7a\x65\
\x6f\x66\x28\x62\x74\x66\x5f\x6c\x6f\x61\x64\x5f\x61\x74\x74\x72\x29\x29\x3b\0\
\x09\x69\x66\x20\x28\x72\x65\x74\x20\x3c\x3d\x20\x30\x29\0\x09\x63\x74\x78\x2d\
\x3e\x62\x74\x66\x5f\x66\x64\x20\x3d\x20\x72\x65\x74\x3b\0\x09\x6d\x61\x70\x5f\
\x63\x72\x65\x61\x74\x65\x5f\x61\x74\x74\x72\x2e\x6d\x61\x78\x5f\x65\x6e\x74\
\x72\x69\x65\x73\x20\x3d\x20\x63\x74\x78\x2d\x3e\x6d\x61\x78\x5f\x65\x6e\x74\
\x72\x69\x65\x73\x3b\0\x09\x6d\x61\x70\x5f\x63\x72\x65\x61\x74\x65\x5f\x61\x74\
\x74\x72\x2e\x62\x74\x66\x5f\x66\x64\x20\x3d\x20\x72\x65\x74\x3b\0\x09\x70\x72\
\x6f\x67\x5f\x6c\x6f\x61\x64\x5f\x61\x74\x74\x72\x2e\x6c\x69\x63\x65\x6e\x73\
\x65\x20\x3d\x20\x70\x74\x72\x5f\x74\x6f\x5f\x75\x36\x34\x28\x6c\x69\x63\x65\
\x6e\x73\x65\x29\x3b\0\x09\x70\x72\x6f\x67\x5f\x6c\x6f\x61\x64\x5f\x61\x74\x74\
\x72\x2e\x69\x6e\x73\x6e\x73\x20\x3d\x20\x70\x74\x72\x5f\x74\x6f\x5f\x75\x36\
\x34\x28\x69\x6e\x73\x6e\x73\x29\x3b\0\x09\x70\x72\x6f\x67\x5f\x6c\x6f\x61\x64\
\x5f\x61\x74\x74\x72\x2e\x6c\x6f\x67\x5f\x62\x75\x66\x20\x3d\x20\x63\x74\x78\
\x2d\x3e\x6c\x6f\x67\x5f\x62\x75\x66\x3b\0\x09\x70\x72\x6f\x67\x5f\x6c\x6f\x61\
\x64\x5f\x61\x74\x74\x72\x2e\x6c\x6f\x67\x5f\x73\x69\x7a\x65\x20\x3d\x20\x63\
\x74\x78\x2d\x3e\x6c\x6f\x67\x5f\x73\x69\x7a\x65\x3b\0\x09\x70\x72\x6f\x67\x5f\
\x6c\x6f\x61\x64\x5f\x61\x74\x74\x72\x2e\x6c\x6f\x67\x5f\x6c\x65\x76\x65\x6c\
\x20\x3d\x20\x31\x3b\0\x09\x72\x65\x74\x20\x3d\x20\x62\x70\x66\x5f\x73\x79\x73\
\x5f\x62\x70\x66\x28\x42\x50\x46\x5f\x4d\x41\x50\x5f\x43\x52\x45\x41\x54\x45\
\x2c\x20\x26\x6d\x61\x70\x5f\x63\x72\x65\x61\x74\x65\x5f\x61\x74\x74\x72\x2c\
\x20\x73\x69\x7a\x65\x6f\x66\x28\x6d\x61\x70\x5f\x63\x72\x65\x61\x74\x65\x5f\
\x61\x74\x74\x72\x29\x29\x3b\0\x09\x63\x74\x78\x2d\x3e\x6d\x61\x70\x5f\x66\x64\
\x20\x3d\x20\x72\x65\x74\x3b\0\x09\x6d\x61\x70\x5f\x75\x70\x64\x61\x74\x65\x5f\
\x61\x74\x74\x72\x2e\x6d\x61\x70\x5f\x66\x64\x20\x3d\x20\x72\x65\x74\x3b\0\x09\
\x6d\x61\x70\x5f\x75\x70\x64\x61\x74\x65\x5f\x61\x74\x74\x72\x2e\x76\x61\x6c\
\x75\x65\x20\x3d\x20\x70\x74\x72\x5f\x74\x6f\x5f\x75\x36\x34\x28\x26\x76\x61\
\x6c\x75\x65\x29\x3b\0\x09\x6d\x61\x70\x5f\x75\x70\x64\x61\x74\x65\x5f\x61\x74\
\x74\x72\x2e\x6b\x65\x79\x20\x3d\x20\x70\x74\x72\x5f\x74\x6f\x5f\x75\x36\x34\
\x28\x26\x6b\x65\x79\x29\x3b\0\x09\x69\x6e\x73\x6e\x73\x5b\x33\x5d\x2e\x69\x6d\
\x6d\x20\x3d\x20\x72\x65\x74\x3b\0\x09\x72\x65\x74\x20\x3d\x20\x62\x70\x66\x5f\
\x73\x79\x73\x5f\x62\x70\x66\x28\x42\x50\x46\x5f\x4d\x41\x50\x5f\x55\x50\x44\
\x41\x54\x45\x5f\x45\x4c\x45\x4d\x2c\x20\x26\x6d\x61\x70\x5f\x75\x70\x64\x61\
\x74\x65\x5f\x61\x74\x74\x72\x2c\x20\x73\x69\x7a\x65\x6f\x66\x28\x6d\x61\x70\
\x5f\x75\x70\x64\x61\x74\x65\x5f\x61\x74\x74\x72\x29\x29\x3b\0\x09\x69\x66\x20\
\x28\x72\x65\x74\x20\x3c\x20\x30\x29\0\x09\x72\x65\x74\x20\x3d\x20\x62\x70\x66\
\x5f\x73\x79\x73\x5f\x62\x70\x66\x28\x42\x50\x46\x5f\x50\x52\x4f\x47\x5f\x4c\
\x4f\x41\x44\x2c\x20\x26\x70\x72\x6f\x67\x5f\x6c\x6f\x61\x64\x5f\x61\x74\x74\
\x72\x2c\x20\x73\x69\x7a\x65\x6f\x66\x28\x70\x72\x6f\x67\x5f\x6c\x6f\x61\x64\
\x5f\x61\x74\x74\x72\x29\x29\x3b\0\x09\x63\x74\x78\x2d\x3e\x70\x72\x6f\x67\x5f\
\x66\x64\x20\x3d\x20\x72\x65\x74\x3b\0\x7d\0\x09\x69\x6e\x74\x20\x7a\x65\x72\
\x6f\x20\x3d\x20\x30\x2c\x20\x72\x65\x74\x20\x3d\x20\x30\x2c\x20\x6f\x75\x74\
\x65\x72\x5f\x66\x64\x20\x3d\x20\x2d\x31\x2c\x20\x69\x6e\x6e\x65\x72\x5f\x66\
\x64\x20\x3d\x20\x2d\x31\x2c\x20\x65\x72\x72\x3b\0\x09\x61\x74\x74\x72\x20\x3d\
\x20\x62\x70\x66\x5f\x6d\x61\x70\x5f\x6c\x6f\x6f\x6b\x75\x70\x5f\x65\x6c\x65\
\x6d\x28\x28\x73\x74\x72\x75\x63\x74\x20\x62\x70\x66\x5f\x6d\x61\x70\x20\x2a\
\x29\x26\x62\x70\x66\x5f\x61\x74\x74\x72\x5f\x61\x72\x72\x61\x79\x2c\x20\x26\
\x7a\x65\x72\x6f\x29\x3b\0\x09\x69\x66\x20\x28\x21\x61\x74\x74\x72\x29\0\x09\
\x6d\x65\x6d\x73\x65\x74\x28\x61\x74\x74\x72\x2c\x20\x30\x2c\x20\x61\x74\x74\
\x72\x5f\x73\x7a\x29\x3b\0\x09\x61\x74\x74\x72\x2d\x3e\x6d\x61\x70\x5f\x69\x64\
\x20\x3d\x20\x28\x28\x73\x74\x72\x75\x63\x74\x20\x62\x70\x66\x5f\x6d\x61\x70\
\x20\x2a\x29\x26\x6f\x75\x74\x65\x72\x5f\x61\x72\x72\x61\x79\x5f\x6d\x61\x70\
\x29\x2d\x3e\x69\x64\x3b\0\x09\x6f\x75\x74\x65\x72\x5f\x66\x64\x20\x3d\x20\x62\
\x70\x66\x5f\x73\x79\x73\x5f\x62\x70\x66\x28\x42\x50\x46\x5f\x4d\x41\x50\x5f\
\x47\x45\x54\x5f\x46\x44\x5f\x42\x59\x5f\x49\x44\x2c\x20\x61\x74\x74\x72\x2c\
\x20\x61\x74\x74\x72\x5f\x73\x7a\x29\x3b\0\x09\x69\x66\x20\x28\x6f\x75\x74\x65\
\x72\x5f\x66\x64\x20\x3c\x20\x30\x29\0\x09\x61\x74\x74\x72\x2d\x3e\x76\x61\x6c\
\x75\x65\x5f\x73\x69\x7a\x65\x20\x3d\x20\x34\x3b\0\x09\x61\x74\x74\x72\x2d\x3e\
\x6d\x61\x70\x5f\x74\x79\x70\x65\x20\x3d\x20\x42\x50\x46\x5f\x4d\x41\x50\x5f\
\x54\x59\x50\x45\x5f\x41\x52\x52\x41\x59\x3b\0\x09\x69\x6e\x6e\x65\x72\x5f\x66\
\x64\x20\x3d\x20\x62\x70\x66\x5f\x73\x79\x73\x5f\x62\x70\x66\x28\x42\x50\x46\
\x5f\x4d\x41\x50\x5f\x43\x52\x45\x41\x54\x45\x2c\x20\x61\x74\x74\x72\x2c\x20\
\x61\x74\x74\x72\x5f\x73\x7a\x29\x3b\0\x09\x69\x66\x20\x28\x69\x6e\x6e\x65\x72\
\x5f\x66\x64\x20\x3c\x20\x30\x29\0\x09\x61\x74\x74\x72\x2d\x3e\x76\x61\x6c\x75\
\x65\x20\x3d\x20\x70\x74\x72\x5f\x74\x6f\x5f\x75\x36\x34\x28\x26\x69\x6e\x6e\
\x65\x72\x5f\x66\x64\x29\x3b\0\x09\x61\x74\x74\x72\x2d\x3e\x6b\x65\x79\x20\x3d\
\x20\x70\x74\x72\x5f\x74\x6f\x5f\x75\x36\x34\x28\x26\x7a\x65\x72\x6f\x29\x3b\0\
\x09\x61\x74\x74\x72\x2d\x3e\x6d\x61\x70\x5f\x66\x64\x20\x3d\x20\x6f\x75\x74\
\x65\x72\x5f\x66\x64\x3b\0\x09\x65\x72\x72\x20\x3d\x20\x62\x70\x66\x5f\x73\x79\
\x73\x5f\x62\x70\x66\x28\x42\x50\x46\x5f\x4d\x41\x50\x5f\x55\x50\x44\x41\x54\
\x45\x5f\x45\x4c\x45\x4d\x2c\x20\x61\x74\x74\x72\x2c\x20\x61\x74\x74\x72\x5f\
\x73\x7a\x29\x3b\0\x09\x69\x66\x20\x28\x65\x72\x72\x29\0\x09\x65\x72\x72\x20\
\x3d\x20\x62\x70\x66\x5f\x73\x79\x73\x5f\x62\x70\x66\x28\x42\x50\x46\x5f\x4d\
\x41\x50\x5f\x44\x45\x4c\x45\x54\x45\x5f\x45\x4c\x45\x4d\x2c\x20\x61\x74\x74\
\x72\x2c\x20\x61\x74\x74\x72\x5f\x73\x7a\x29\x3b\0\x09\x69\x66\x20\x28\x69\x6e\
\x6e\x65\x72\x5f\x66\x64\x20\x3e\x3d\x20\x30\x29\0\x09\x09\x62\x70\x66\x5f\x73\
\x79\x73\x5f\x63\x6c\x6f\x73\x65\x28\x69\x6e\x6e\x65\x72\x5f\x66\x64\x29\x3b\0\
\x09\x69\x66\x20\x28\x6f\x75\x74\x65\x72\x5f\x66\x64\x20\x3e\x3d\x20\x30\x29\0\
\x09\x09\x62\x70\x66\x5f\x73\x79\x73\x5f\x63\x6c\x6f\x73\x65\x28\x6f\x75\x74\
\x65\x72\x5f\x66\x64\x29\x3b\0\x09\x72\x65\x74\x75\x72\x6e\x20\x72\x65\x74\x3b\
\0\x30\x3a\x30\0\x2e\x6d\x61\x70\x73\0\x2e\x64\x61\x74\x61\0\x2e\x72\x6f\x64\
\x61\x74\x61\0\x73\x79\x73\x63\x61\x6c\x6c\0\0\0\0\0\x9f\xeb\x01\0\x20\0\0\0\0\
\0\0\0\x1c\0\0\0\x1c\0\0\0\x3c\x04\0\0\x58\x04\0\0\x1c\0\0\0\x08\0\0\0\xd8\x0e\
\0\0\x02\0\0\0\0\0\0\0\x44\0\0\0\x98\x02\0\0\x47\0\0\0\x10\0\0\0\xd8\x0e\0\0\
\x43\0\0\0\0\0\0\0\xb0\x07\0\0\x31\x08\0\0\0\x88\x01\0\x08\0\0\0\xb0\x07\0\0\
\x51\x08\0\0\x14\x74\x01\0\x20\0\0\0\xb0\x07\0\0\0\0\0\0\0\0\0\0\x28\0\0\0\xb0\
\x07\0\0\x51\x08\0\0\x14\x74\x01\0\x40\0\0\0\xb0\x07\0\0\x76\x08\0\0\x04\x24\
\x01\0\xd0\0\0\0\xb0\x07\0\0\x85\x08\0\0\x09\x78\x01\0\xf8\0\0\0\xb0\x07\0\0\
\xcf\x08\0\0\x06\xfc\x01\0\0\x01\0\0\xb0\x07\0\0\xde\x08\0\0\x0e\x08\x02\0\x08\
\x01\0\0\xb0\x07\0\0\xf2\x08\0\0\x25\x0c\x02\0\x10\x01\0\0\xb0\x07\0\0\xf2\x08\
\0\0\x1e\x0c\x02\0\x20\x01\0\0\xb0\x07\0\0\x23\x09\0\0\x19\x10\x02\0\x28\x01\0\
\0\xb0\x07\0\0\xf2\x08\0\0\x1e\x0c\x02\0\x30\x01\0\0\xb0\x07\0\0\x42\x09\0\0\
\x19\x18\x02\0\x58\x01\0\0\xb0\x07\0\0\x71\x09\0\0\x17\x1c\x02\0\x70\x01\0\0\
\xb0\x07\0\0\x9c\x09\0\0\x20\x20\x02\0\x78\x01\0\0\xb0\x07\0\0\x9c\x09\0\0\x19\
\x20\x02\0\x80\x01\0\0\xb0\x07\0\0\xc4\x09\0\0\x21\x24\x02\0\x88\x01\0\0\xb0\
\x07\0\0\xc4\x09\0\0\x1a\x24\x02\0\x90\x01\0\0\xb0\x07\0\0\xee\x09\0\0\x1b\x28\
\x02\0\x98\x01\0\0\xb0\x07\0\0\x0d\x0a\0\0\x08\x30\x02\0\xc0\x01\0\0\xb0\x07\0\
\0\xcf\x08\0\0\x06\x34\x02\0\xc8\x01\0\0\xb0\x07\0\0\x5c\x0a\0\0\x0e\x3c\x02\0\
\xd0\x01\0\0\xb0\x07\0\0\x70\x0a\0\0\x19\x48\x02\0\xe0\x01\0\0\xb0\x07\0\0\x8f\
\x0a\0\0\x18\x50\x02\0\xf8\x01\0\0\xb0\x07\0\0\xbc\x0a\0\0\x16\x4c\x02\0\x10\
\x02\0\0\xb0\x07\0\0\xe5\x0a\0\0\x0f\x40\x02\0\x18\x02\0\0\xb0\x07\0\0\x70\x0a\
\0\0\x19\x48\x02\0\x20\x02\0\0\xb0\x07\0\0\xfa\x0a\0\0\x08\x54\x02\0\x48\x02\0\
\0\xb0\x07\0\0\x4e\x0b\0\0\x06\x58\x02\0\x50\x02\0\0\xb0\x07\0\0\x5c\x0b\0\0\
\x08\x64\x02\0\x78\x02\0\0\xb0\x07\0\0\xcf\x08\0\0\x06\x68\x02\0\x80\x02\0\0\
\xb0\x07\0\0\xa8\x0b\0\0\x0f\x70\x02\0\x90\x02\0\0\xb0\x07\0\0\xbd\x0b\0\0\x01\
\x78\x02\0\x98\x02\0\0\xb0\x07\0\0\xbf\x0b\0\0\x06\x8c\x02\0\xa0\x02\0\0\xb0\
\x07\0\0\xbf\x0b\0\0\x28\x8c\x02\0\xb0\x02\0\0\xb0\x07\0\0\xbf\x0b\0\0\x06\x8c\
\x02\0\xb8\x02\0\0\xb0\x07\0\0\xfa\x0b\0\0\x09\x9c\x02\0\xe8\x02\0\0\xb0\x07\0\
\0\x41\x0c\0\0\x06\xa0\x02\0\xf0\x02\0\0\xb0\x07\0\0\x4d\x0c\0\0\x02\xac\x02\0\
\x90\x03\0\0\xb0\x07\0\0\x68\x0c\0\0\x37\xb0\x02\0\x98\x03\0\0\xb0\x07\0\0\x68\
\x0c\0\0\x0f\xb0\x02\0\xa0\x03\0\0\xb0\x07\0\0\xa2\x0c\0\0\x0d\xb4\x02\0\xc8\
\x03\0\0\xb0\x07\0\0\xe0\x0c\0\0\x06\xb8\x02\0\xe0\x03\0\0\xb0\x07\0\0\xf3\x0c\
\0\0\x13\xd0\x02\0\xf8\x03\0\0\xb0\x07\0\0\x0a\x0d\0\0\x11\xc8\x02\0\0\x04\0\0\
\xb0\x07\0\0\x4d\x0c\0\0\x02\xc4\x02\0\x80\x04\0\0\xb0\x07\0\0\x30\x0d\0\0\x0d\
\xd8\x02\0\xa0\x04\0\0\xb0\x07\0\0\x30\x0d\0\0\x0b\xd8\x02\0\xa8\x04\0\0\xb0\
\x07\0\0\x68\x0d\0\0\x06\xdc\x02\0\xb0\x04\0\0\xb0\x07\0\0\x4d\x0c\0\0\x02\xe8\
\x02\0\x50\x05\0\0\xb0\x07\0\0\x7b\x0d\0\0\x0e\xf4\x02\0\x60\x05\0\0\xb0\x07\0\
\0\x4d\x0c\0\0\x02\xe8\x02\0\x68\x05\0\0\xb0\x07\0\0\xa1\x0d\0\0\x0c\xf0\x02\0\
\x70\x05\0\0\xb0\x07\0\0\xc1\x0d\0\0\x0f\xec\x02\0\x78\x05\0\0\xb0\x07\0\0\xdb\
\x0d\0\0\x08\xf8\x02\0\x98\x05\0\0\xb0\x07\0\0\x13\x0e\0\0\x06\xfc\x02\0\xa0\
\x05\0\0\xb0\x07\0\0\x4d\x0c\0\0\x02\x08\x03\0\x40\x06\0\0\xb0\x07\0\0\xa1\x0d\
\0\0\x0c\x10\x03\0\x48\x06\0\0\xb0\x07\0\0\xc1\x0d\0\0\x0f\x0c\x03\0\x50\x06\0\
\0\xb0\x07\0\0\x1d\x0e\0\0\x08\x14\x03\0\x70\x06\0\0\xb0\x07\0\0\x13\x0e\0\0\
\x06\x18\x03\0\x80\x06\0\0\xb0\x07\0\0\x55\x0e\0\0\x06\x28\x03\0\x88\x06\0\0\
\xb0\x07\0\0\x55\x0e\0\0\x06\x28\x03\0\x90\x06\0\0\xb0\x07\0\0\x69\x0e\0\0\x03\
\x2c\x03\0\x98\x06\0\0\xb0\x07\0\0\x84\x0e\0\0\x06\x30\x03\0\xa0\x06\0\0\xb0\
\x07\0\0\x98\x0e\0\0\x03\x34\x03\0\xb0\x06\0\0\xb0\x07\0\0\xb3\x0e\0\0\x02\x38\
\x03\0\x10\0\0\0\xd8\x0e\0\0\x01\0\0\0\x90\x03\0\0\x48\0\0\0\xc0\x0e\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\x03\0\0\
\0\x20\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x40\0\0\0\0\0\0\0\x78\x01\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x09\0\0\0\x02\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\xb8\x01\0\0\0\0\0\0\x28\x02\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\
\x08\0\0\0\0\0\0\0\x18\0\0\0\0\0\0\0\x11\0\0\0\x01\0\0\0\x06\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\xe0\x03\0\0\0\0\0\0\xc0\x06\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\x19\0\0\0\x01\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\xa0\x0a\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\x21\0\0\0\x01\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xa8\x0a\0\0\0\
\0\0\0\x68\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x27\
\0\0\0\x01\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x10\x0b\0\0\0\0\0\0\x98\x02\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x2d\0\0\0\x01\0\
\0\0\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xa8\x0d\0\0\0\0\0\0\x3c\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x54\x01\0\0\x09\0\0\0\x40\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\xe8\x0d\0\0\0\0\0\0\xe0\0\0\0\0\0\0\0\x02\0\0\0\x03\
\0\0\0\x08\0\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\x60\x01\0\0\x09\0\0\0\x40\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\xc8\x0e\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\x02\0\0\0\x05\0\0\0\
\x08\0\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\x6a\x01\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\xd8\x0e\0\0\0\0\0\0\x14\x1f\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\x6f\x01\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\xf0\x2d\0\0\0\0\0\0\x94\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0";

	*sz = sizeof(data) - 1;
	return (const void *)data;
}

#ifdef __cplusplus
struct syscall *syscall::open(const struct bpf_object_open_opts *opts) { return syscall__open_opts(opts); }
struct syscall *syscall::open_and_load() { return syscall__open_and_load(); }
int syscall::load(struct syscall *skel) { return syscall__load(skel); }
int syscall::attach(struct syscall *skel) { return syscall__attach(skel); }
void syscall::detach(struct syscall *skel) { syscall__detach(skel); }
void syscall::destroy(struct syscall *skel) { syscall__destroy(skel); }
const void *syscall::elf_bytes(size_t *sz) { return syscall__elf_bytes(sz); }
#endif /* __cplusplus */

__attribute__((unused)) static void
syscall__assert(struct syscall *s __attribute__((unused)))
{
#ifdef __cplusplus
#define _Static_assert static_assert
#endif
#ifdef __cplusplus
#undef _Static_assert
#endif
}

#endif /* __SYSCALL_SKEL_H__ */
