/* SPDX-License-Identifier: (LGPL-2.1 OR BSD-2-Clause) */

/* THIS FILE IS AUTOGENERATED BY BPFTOOL! */
#ifndef __LSM_SKEL_H__
#define __LSM_SKEL_H__

#include <errno.h>
#include <stdlib.h>
#include <bpf/libbpf.h>

struct lsm {
	struct bpf_object_skeleton *skeleton;
	struct bpf_object *obj;
	struct {
		struct bpf_map *inner_map;
		struct bpf_map *outer_arr;
		struct bpf_map *outer_hash;
		struct bpf_map *array;
		struct bpf_map *hash;
		struct bpf_map *lru_hash;
		struct bpf_map *percpu_array;
		struct bpf_map *percpu_hash;
		struct bpf_map *lru_percpu_hash;
		struct bpf_map *bss;
	} maps;
	struct {
		struct bpf_program *test_int_hook;
		struct bpf_program *test_void_hook;
		struct bpf_program *test_task_free;
		struct bpf_program *test_sys_setdomainname;
	} progs;
	struct {
		struct bpf_link *test_int_hook;
		struct bpf_link *test_void_hook;
		struct bpf_link *test_task_free;
		struct bpf_link *test_sys_setdomainname;
	} links;
	struct lsm__bss {
		int monitored_pid;
		int mprotect_count;
		int bprm_count;
		int copy_test;
	} *bss;

#ifdef __cplusplus
	static inline struct lsm *open(const struct bpf_object_open_opts *opts = nullptr);
	static inline struct lsm *open_and_load();
	static inline int load(struct lsm *skel);
	static inline int attach(struct lsm *skel);
	static inline void detach(struct lsm *skel);
	static inline void destroy(struct lsm *skel);
	static inline const void *elf_bytes(size_t *sz);
#endif /* __cplusplus */
};

static void
lsm__destroy(struct lsm *obj)
{
	if (!obj)
		return;
	if (obj->skeleton)
		bpf_object__destroy_skeleton(obj->skeleton);
	free(obj);
}

static inline int
lsm__create_skeleton(struct lsm *obj);

static inline struct lsm *
lsm__open_opts(const struct bpf_object_open_opts *opts)
{
	struct lsm *obj;
	int err;

	obj = (struct lsm *)calloc(1, sizeof(*obj));
	if (!obj) {
		errno = ENOMEM;
		return NULL;
	}

	err = lsm__create_skeleton(obj);
	if (err)
		goto err_out;

	err = bpf_object__open_skeleton(obj->skeleton, opts);
	if (err)
		goto err_out;

	return obj;
err_out:
	lsm__destroy(obj);
	errno = -err;
	return NULL;
}

static inline struct lsm *
lsm__open(void)
{
	return lsm__open_opts(NULL);
}

static inline int
lsm__load(struct lsm *obj)
{
	return bpf_object__load_skeleton(obj->skeleton);
}

static inline struct lsm *
lsm__open_and_load(void)
{
	struct lsm *obj;
	int err;

	obj = lsm__open();
	if (!obj)
		return NULL;
	err = lsm__load(obj);
	if (err) {
		lsm__destroy(obj);
		errno = -err;
		return NULL;
	}
	return obj;
}

static inline int
lsm__attach(struct lsm *obj)
{
	return bpf_object__attach_skeleton(obj->skeleton);
}

static inline void
lsm__detach(struct lsm *obj)
{
	bpf_object__detach_skeleton(obj->skeleton);
}

static inline const void *lsm__elf_bytes(size_t *sz);

static inline int
lsm__create_skeleton(struct lsm *obj)
{
	struct bpf_object_skeleton *s;
	int err;

	s = (struct bpf_object_skeleton *)calloc(1, sizeof(*s));
	if (!s)	{
		err = -ENOMEM;
		goto err;
	}

	s->sz = sizeof(*s);
	s->name = "lsm";
	s->obj = &obj->obj;

	/* maps */
	s->map_cnt = 10;
	s->map_skel_sz = sizeof(*s->maps);
	s->maps = (struct bpf_map_skeleton *)calloc(s->map_cnt, s->map_skel_sz);
	if (!s->maps) {
		err = -ENOMEM;
		goto err;
	}

	s->maps[0].name = "inner_map";
	s->maps[0].map = &obj->maps.inner_map;

	s->maps[1].name = "outer_arr";
	s->maps[1].map = &obj->maps.outer_arr;

	s->maps[2].name = "outer_hash";
	s->maps[2].map = &obj->maps.outer_hash;

	s->maps[3].name = "array";
	s->maps[3].map = &obj->maps.array;

	s->maps[4].name = "hash";
	s->maps[4].map = &obj->maps.hash;

	s->maps[5].name = "lru_hash";
	s->maps[5].map = &obj->maps.lru_hash;

	s->maps[6].name = "percpu_array";
	s->maps[6].map = &obj->maps.percpu_array;

	s->maps[7].name = "percpu_hash";
	s->maps[7].map = &obj->maps.percpu_hash;

	s->maps[8].name = "lru_percpu_hash";
	s->maps[8].map = &obj->maps.lru_percpu_hash;

	s->maps[9].name = "lsm.bss";
	s->maps[9].map = &obj->maps.bss;
	s->maps[9].mmaped = (void **)&obj->bss;

	/* programs */
	s->prog_cnt = 4;
	s->prog_skel_sz = sizeof(*s->progs);
	s->progs = (struct bpf_prog_skeleton *)calloc(s->prog_cnt, s->prog_skel_sz);
	if (!s->progs) {
		err = -ENOMEM;
		goto err;
	}

	s->progs[0].name = "test_int_hook";
	s->progs[0].prog = &obj->progs.test_int_hook;
	s->progs[0].link = &obj->links.test_int_hook;

	s->progs[1].name = "test_void_hook";
	s->progs[1].prog = &obj->progs.test_void_hook;
	s->progs[1].link = &obj->links.test_void_hook;

	s->progs[2].name = "test_task_free";
	s->progs[2].prog = &obj->progs.test_task_free;
	s->progs[2].link = &obj->links.test_task_free;

	s->progs[3].name = "test_sys_setdomainname";
	s->progs[3].prog = &obj->progs.test_sys_setdomainname;
	s->progs[3].link = &obj->links.test_sys_setdomainname;

	s->data = lsm__elf_bytes(&s->data_sz);

	obj->skeleton = s;
	return 0;
err:
	bpf_object__destroy_skeleton(s);
	return err;
}

static inline const void *lsm__elf_bytes(size_t *sz)
{
	static const char data[] __attribute__((__aligned__(8))) = "\
\x7f\x45\x4c\x46\x02\x01\x01\0\0\0\0\0\0\0\0\0\x01\0\xf7\0\x01\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x28\x3f\0\0\0\0\0\0\0\0\0\0\x40\0\0\0\0\0\x40\0\x10\0\
\x01\0\0\x2e\x73\x74\x72\x74\x61\x62\0\x2e\x73\x79\x6d\x74\x61\x62\0\x6c\x73\
\x6d\x2f\x66\x69\x6c\x65\x5f\x6d\x70\x72\x6f\x74\x65\x63\x74\0\x6c\x73\x6d\x2e\
\x73\x2f\x62\x70\x72\x6d\x5f\x63\x6f\x6d\x6d\x69\x74\x74\x65\x64\x5f\x63\x72\
\x65\x64\x73\0\x6c\x73\x6d\x2f\x74\x61\x73\x6b\x5f\x66\x72\x65\x65\0\x66\x65\
\x6e\x74\x72\x79\x2e\x73\x2f\x5f\x5f\x61\x72\x6d\x36\x34\x5f\x73\x79\x73\x5f\
\x73\x65\x74\x64\x6f\x6d\x61\x69\x6e\x6e\x61\x6d\x65\0\x2e\x6d\x61\x70\x73\0\
\x6c\x69\x63\x65\x6e\x73\x65\0\x2e\x62\x73\x73\0\x6c\x73\x6d\x2e\x63\0\x4c\x42\
\x42\x30\x5f\x35\0\x4c\x42\x42\x31\x5f\x32\0\x4c\x42\x42\x31\x5f\x34\0\x4c\x42\
\x42\x31\x5f\x36\0\x4c\x42\x42\x31\x5f\x38\0\x4c\x42\x42\x31\x5f\x31\x30\0\x4c\
\x42\x42\x31\x5f\x31\x32\0\x4c\x42\x42\x31\x5f\x31\x34\0\x4c\x42\x42\x31\x5f\
\x31\x37\0\x4c\x42\x42\x31\x5f\x32\x30\0\x4c\x42\x42\x33\x5f\x33\0\x4c\x42\x42\
\x33\x5f\x35\0\x4c\x42\x42\x33\x5f\x36\0\x74\x65\x73\x74\x5f\x69\x6e\x74\x5f\
\x68\x6f\x6f\x6b\0\x6d\x6f\x6e\x69\x74\x6f\x72\x65\x64\x5f\x70\x69\x64\0\x6d\
\x70\x72\x6f\x74\x65\x63\x74\x5f\x63\x6f\x75\x6e\x74\0\x74\x65\x73\x74\x5f\x76\
\x6f\x69\x64\x5f\x68\x6f\x6f\x6b\0\x62\x70\x72\x6d\x5f\x63\x6f\x75\x6e\x74\0\
\x61\x72\x72\x61\x79\0\x68\x61\x73\x68\0\x6c\x72\x75\x5f\x68\x61\x73\x68\0\x70\
\x65\x72\x63\x70\x75\x5f\x61\x72\x72\x61\x79\0\x70\x65\x72\x63\x70\x75\x5f\x68\
\x61\x73\x68\0\x6c\x72\x75\x5f\x70\x65\x72\x63\x70\x75\x5f\x68\x61\x73\x68\0\
\x6f\x75\x74\x65\x72\x5f\x61\x72\x72\0\x6f\x75\x74\x65\x72\x5f\x68\x61\x73\x68\
\0\x74\x65\x73\x74\x5f\x74\x61\x73\x6b\x5f\x66\x72\x65\x65\0\x74\x65\x73\x74\
\x5f\x73\x79\x73\x5f\x73\x65\x74\x64\x6f\x6d\x61\x69\x6e\x6e\x61\x6d\x65\0\x63\
\x6f\x70\x79\x5f\x74\x65\x73\x74\0\x69\x6e\x6e\x65\x72\x5f\x6d\x61\x70\0\x5f\
\x6c\x69\x63\x65\x6e\x73\x65\0\x2e\x72\x65\x6c\x6c\x73\x6d\x2f\x66\x69\x6c\x65\
\x5f\x6d\x70\x72\x6f\x74\x65\x63\x74\0\x2e\x72\x65\x6c\x6c\x73\x6d\x2e\x73\x2f\
\x62\x70\x72\x6d\x5f\x63\x6f\x6d\x6d\x69\x74\x74\x65\x64\x5f\x63\x72\x65\x64\
\x73\0\x2e\x72\x65\x6c\x66\x65\x6e\x74\x72\x79\x2e\x73\x2f\x5f\x5f\x61\x72\x6d\
\x36\x34\x5f\x73\x79\x73\x5f\x73\x65\x74\x64\x6f\x6d\x61\x69\x6e\x6e\x61\x6d\
\x65\0\x2e\x72\x65\x6c\x2e\x6d\x61\x70\x73\0\x2e\x42\x54\x46\0\x2e\x42\x54\x46\
\x2e\x65\x78\x74\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x82\0\0\
\0\x04\0\xf1\xff\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x03\0\x03\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x88\0\0\0\0\0\x03\0\xc8\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\x03\0\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x8f\0\0\0\0\0\x04\0\x70\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\x96\0\0\0\0\0\x04\0\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\x9d\0\0\0\0\0\x04\0\x58\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xa4\0\0\0\0\0\x04\
\0\x98\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xab\0\0\0\0\0\x04\0\xd8\x01\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\xb3\0\0\0\0\0\x04\0\x18\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xbb\0\
\0\0\0\0\x04\0\x58\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xc3\0\0\0\0\0\x04\0\xc0\x02\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xcb\0\0\0\0\0\x04\0\x28\x03\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\x03\0\x05\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x03\0\x06\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xd3\0\0\0\0\0\x06\0\xa0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\xda\0\0\0\0\0\x06\0\xb8\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xe1\0\0\0\0\0\
\x06\0\xe0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xe8\0\0\0\x12\0\x03\0\0\0\0\0\0\0\0\0\
\xd0\0\0\0\0\0\0\0\xf6\0\0\0\x11\0\x09\0\0\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\x04\
\x01\0\0\x11\0\x09\0\x04\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\x13\x01\0\0\x12\0\x04\
\0\0\0\0\0\0\0\0\0\x38\x03\0\0\0\0\0\0\x22\x01\0\0\x11\0\x09\0\x08\0\0\0\0\0\0\
\0\x04\0\0\0\0\0\0\0\x2d\x01\0\0\x11\0\x07\0\x68\0\0\0\0\0\0\0\x20\0\0\0\0\0\0\
\0\x33\x01\0\0\x11\0\x07\0\x88\0\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\x38\x01\0\0\x11\
\0\x07\0\xa8\0\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\x41\x01\0\0\x11\0\x07\0\xc8\0\0\0\
\0\0\0\0\x20\0\0\0\0\0\0\0\x4e\x01\0\0\x11\0\x07\0\xe8\0\0\0\0\0\0\0\x20\0\0\0\
\0\0\0\0\x5a\x01\0\0\x11\0\x07\0\x08\x01\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\x6a\x01\
\0\0\x11\0\x07\0\x20\0\0\0\0\0\0\0\x28\0\0\0\0\0\0\0\x74\x01\0\0\x11\0\x07\0\
\x48\0\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\x7f\x01\0\0\x12\0\x05\0\0\0\0\0\0\0\0\0\
\x10\0\0\0\0\0\0\0\x8e\x01\0\0\x12\0\x06\0\0\0\0\0\0\0\0\0\xf0\0\0\0\0\0\0\0\
\xa5\x01\0\0\x11\0\x09\0\x0c\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\xaf\x01\0\0\x11\0\
\x07\0\0\0\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\xb9\x01\0\0\x11\0\x08\0\0\0\0\0\0\0\0\
\0\x04\0\0\0\0\0\0\0\x79\x10\x18\0\0\0\0\0\x56\0\x17\0\0\0\0\0\x79\x16\0\0\0\0\
\0\0\x85\0\0\0\x0e\0\0\0\xbf\x01\0\0\0\0\0\0\xb4\0\0\0\0\0\0\0\x79\x62\x10\0\0\
\0\0\0\x79\x63\0\0\0\0\0\0\x79\x22\x18\x01\0\0\0\0\x2d\x23\x0f\0\0\0\0\0\x79\
\x62\x10\0\0\0\0\0\x79\x22\x18\x01\0\0\0\0\x79\x63\x08\0\0\0\0\0\xad\x23\x0b\0\
\0\0\0\0\x18\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x61\x22\0\0\0\0\0\0\x77\x01\0\0\
\x20\0\0\0\x5e\x12\x06\0\0\0\0\0\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x61\x12\0\
\0\0\0\0\0\x04\x02\0\0\x01\0\0\0\x63\x21\0\0\0\0\0\0\xb4\0\0\0\xff\xff\xff\xff\
\x95\0\0\0\0\0\0\0\x79\x17\0\0\0\0\0\0\x85\0\0\0\x0e\0\0\0\xb4\x01\0\0\0\0\0\0\
\x63\x1a\xbc\xff\0\0\0\0\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x61\x11\0\0\0\0\0\
\0\x77\0\0\0\x20\0\0\0\x5e\x01\x05\0\0\0\0\0\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\x61\x12\0\0\0\0\0\0\x04\x02\0\0\x01\0\0\0\x63\x21\0\0\0\0\0\0\x79\x71\0\0\0\
\0\0\0\x79\x11\x10\0\0\0\0\0\x79\x13\x20\x01\0\0\0\0\xbf\xa6\0\0\0\0\0\0\x07\
\x06\0\0\xc0\xff\xff\xff\xbf\x61\0\0\0\0\0\0\xb4\x02\0\0\x40\0\0\0\x85\0\0\0\
\x94\0\0\0\x79\x71\x10\0\0\0\0\0\x79\x13\x20\x01\0\0\0\0\xbf\x61\0\0\0\0\0\0\
\xb4\x02\0\0\x40\0\0\0\x85\0\0\0\x94\0\0\0\xbf\xa2\0\0\0\0\0\0\x07\x02\0\0\xbc\
\xff\xff\xff\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x85\0\0\0\x01\0\0\0\x15\0\x02\
\0\0\0\0\0\xb7\x01\0\0\0\0\0\0\x7b\x10\0\0\0\0\0\0\xbf\xa2\0\0\0\0\0\0\x07\x02\
\0\0\xbc\xff\xff\xff\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x85\0\0\0\x01\0\0\0\
\x15\0\x02\0\0\0\0\0\xb7\x01\0\0\0\0\0\0\x7b\x10\0\0\0\0\0\0\xbf\xa2\0\0\0\0\0\
\0\x07\x02\0\0\xbc\xff\xff\xff\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x85\0\0\0\
\x01\0\0\0\x15\0\x02\0\0\0\0\0\xb7\x01\0\0\0\0\0\0\x7b\x10\0\0\0\0\0\0\xbf\xa2\
\0\0\0\0\0\0\x07\x02\0\0\xbc\xff\xff\xff\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\x85\0\0\0\x01\0\0\0\x15\0\x02\0\0\0\0\0\xb7\x01\0\0\0\0\0\0\x7b\x10\0\0\0\0\0\
\0\xbf\xa2\0\0\0\0\0\0\x07\x02\0\0\xbc\xff\xff\xff\x18\x01\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\x85\0\0\0\x01\0\0\0\x15\0\x02\0\0\0\0\0\xb7\x01\0\0\0\0\0\0\x7b\x10\0\
\0\0\0\0\0\xbf\xa2\0\0\0\0\0\0\x07\x02\0\0\xbc\xff\xff\xff\x18\x01\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\x85\0\0\0\x01\0\0\0\x15\0\x02\0\0\0\0\0\xb7\x01\0\0\0\0\0\0\
\x7b\x10\0\0\0\0\0\0\xbf\xa2\0\0\0\0\0\0\x07\x02\0\0\xbc\xff\xff\xff\x18\x01\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\x85\0\0\0\x01\0\0\0\x15\0\x07\0\0\0\0\0\xbf\xa2\0\0\
\0\0\0\0\x07\x02\0\0\xbc\xff\xff\xff\xbf\x01\0\0\0\0\0\0\x85\0\0\0\x01\0\0\0\
\x15\0\x02\0\0\0\0\0\xb7\x01\0\0\0\0\0\0\x7b\x10\0\0\0\0\0\0\xbf\xa2\0\0\0\0\0\
\0\x07\x02\0\0\xbc\xff\xff\xff\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x85\0\0\0\
\x01\0\0\0\x15\0\x07\0\0\0\0\0\xbf\xa2\0\0\0\0\0\0\x07\x02\0\0\xbc\xff\xff\xff\
\xbf\x01\0\0\0\0\0\0\x85\0\0\0\x01\0\0\0\x15\0\x02\0\0\0\0\0\xb7\x01\0\0\0\0\0\
\0\x7b\x10\0\0\0\0\0\0\xb4\0\0\0\0\0\0\0\x95\0\0\0\0\0\0\0\xb4\0\0\0\0\0\0\0\
\x95\0\0\0\0\0\0\0\xb7\x02\0\0\0\0\0\0\x79\x16\0\0\0\0\0\0\xbf\x63\0\0\0\0\0\0\
\x0f\x23\0\0\0\0\0\0\xbf\xa1\0\0\0\0\0\0\x07\x01\0\0\xf8\xff\xff\xff\xb4\x02\0\
\0\x08\0\0\0\x85\0\0\0\x71\0\0\0\x79\xa3\xf8\xff\0\0\0\0\x79\x66\x08\0\0\0\0\0\
\xb4\x01\0\0\0\0\0\0\x63\x1a\xf4\xff\0\0\0\0\xbf\xa1\0\0\0\0\0\0\x07\x01\0\0\
\xf4\xff\xff\xff\xb4\x02\0\0\x04\0\0\0\x85\0\0\0\x94\0\0\0\x56\x06\x03\0\xfe\
\xff\xff\xff\x55\0\x02\0\0\0\0\0\x61\xa1\xf4\xff\0\0\0\0\x16\x01\x03\0\xd2\x04\
\0\0\x54\x06\0\0\xfe\xff\xff\xff\x56\x06\x06\0\xfc\xff\xff\xff\x55\0\x05\0\xf2\
\xff\xff\xff\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x61\x12\0\0\0\0\0\0\x04\x02\0\
\0\x01\0\0\0\x63\x21\0\0\0\0\0\0\xb4\0\0\0\0\0\0\0\x95\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x47\x50\x4c\0\0\0\0\0\x70\0\0\0\0\0\0\0\
\x01\0\0\0\x14\0\0\0\x98\0\0\0\0\0\0\0\x01\0\0\0\x15\0\0\0\x20\0\0\0\0\0\0\0\
\x01\0\0\0\x14\0\0\0\x48\0\0\0\0\0\0\0\x01\0\0\0\x17\0\0\0\xe8\0\0\0\0\0\0\0\
\x01\0\0\0\x18\0\0\0\x28\x01\0\0\0\0\0\0\x01\0\0\0\x19\0\0\0\x68\x01\0\0\0\0\0\
\0\x01\0\0\0\x1a\0\0\0\xa8\x01\0\0\0\0\0\0\x01\0\0\0\x1b\0\0\0\xe8\x01\0\0\0\0\
\0\0\x01\0\0\0\x1c\0\0\0\x28\x02\0\0\0\0\0\0\x01\0\0\0\x1d\0\0\0\x68\x02\0\0\0\
\0\0\0\x01\0\0\0\x1e\0\0\0\xd0\x02\0\0\0\0\0\0\x01\0\0\0\x1f\0\0\0\xb8\0\0\0\0\
\0\0\0\x01\0\0\0\x22\0\0\0\x40\0\0\0\0\0\0\0\x02\0\0\0\x23\0\0\0\x60\0\0\0\0\0\
\0\0\x02\0\0\0\x23\0\0\0\x9f\xeb\x01\0\x18\0\0\0\0\0\0\0\x0c\x19\0\0\x0c\x19\0\
\0\xbd\x12\0\0\0\0\0\0\0\0\0\x02\x03\0\0\0\x01\0\0\0\0\0\0\x01\x04\0\0\0\x20\0\
\0\x01\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x02\0\0\0\x05\0\0\0\0\0\0\
\x01\x04\0\0\0\x20\0\0\0\0\0\0\0\0\0\0\x02\x06\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\
\x02\0\0\0\x04\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x02\0\0\0\0\0\0\0\0\0\0\x02\
\x09\0\0\0\x19\0\0\0\0\0\0\x08\x0a\0\0\0\x1f\0\0\0\0\0\0\x01\x08\0\0\0\x40\0\0\
\0\x32\0\0\0\x04\0\0\x04\x20\0\0\0\x3c\0\0\0\x01\0\0\0\0\0\0\0\x41\0\0\0\x05\0\
\0\0\x40\0\0\0\x4d\0\0\0\x07\0\0\0\x80\0\0\0\x51\0\0\0\x08\0\0\0\xc0\0\0\0\x32\
\0\0\0\0\0\0\x0e\x0b\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x0e\0\0\0\0\0\0\0\0\0\0\
\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x0c\0\0\0\0\0\0\0\0\0\0\x02\x10\0\0\0\0\0\0\0\
\0\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x04\0\0\0\0\0\0\0\0\0\0\x02\x0b\0\0\0\0\
\0\0\0\0\0\0\x03\0\0\0\0\x11\0\0\0\x04\0\0\0\0\0\0\0\x57\0\0\0\x05\0\0\x04\x20\
\0\0\0\x3c\0\0\0\x0d\0\0\0\0\0\0\0\x41\0\0\0\x05\0\0\0\x40\0\0\0\x61\0\0\0\x0f\
\0\0\0\x80\0\0\0\x6a\0\0\0\x0f\0\0\0\xc0\0\0\0\x75\0\0\0\x12\0\0\0\0\x01\0\0\
\x57\0\0\0\0\0\0\x0e\x13\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x16\0\0\0\0\0\0\0\0\
\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x0d\0\0\0\x7c\0\0\0\x04\0\0\x04\x18\0\0\0\
\x3c\0\0\0\x15\0\0\0\0\0\0\0\x41\0\0\0\x05\0\0\0\x40\0\0\0\x61\0\0\0\x0f\0\0\0\
\x80\0\0\0\x75\0\0\0\x12\0\0\0\xc0\0\0\0\x7c\0\0\0\0\0\0\x0e\x17\0\0\0\x01\0\0\
\0\0\0\0\0\0\0\0\x02\x1a\0\0\0\x87\0\0\0\0\0\0\x08\x1b\0\0\0\x8d\0\0\0\0\0\0\
\x01\x04\0\0\0\x20\0\0\0\0\0\0\0\x04\0\0\x04\x20\0\0\0\x3c\0\0\0\x01\0\0\0\0\0\
\0\0\x41\0\0\0\x05\0\0\0\x40\0\0\0\x4d\0\0\0\x19\0\0\0\x80\0\0\0\x51\0\0\0\x08\
\0\0\0\xc0\0\0\0\x9a\0\0\0\0\0\0\x0e\x1c\0\0\0\x01\0\0\0\0\0\0\0\x04\0\0\x04\
\x20\0\0\0\x3c\0\0\0\x05\0\0\0\0\0\0\0\x41\0\0\0\x05\0\0\0\x40\0\0\0\x4d\0\0\0\
\x19\0\0\0\x80\0\0\0\x51\0\0\0\x08\0\0\0\xc0\0\0\0\xa0\0\0\0\0\0\0\x0e\x1e\0\0\
\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x21\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\
\x04\0\0\0\x09\0\0\0\0\0\0\0\x04\0\0\x04\x20\0\0\0\x3c\0\0\0\x20\0\0\0\0\0\0\0\
\x41\0\0\0\x05\0\0\0\x40\0\0\0\x4d\0\0\0\x19\0\0\0\x80\0\0\0\x51\0\0\0\x08\0\0\
\0\xc0\0\0\0\xa5\0\0\0\0\0\0\x0e\x22\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x25\0\0\
\0\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x06\0\0\0\0\0\0\0\x04\0\0\x04\
\x20\0\0\0\x3c\0\0\0\x24\0\0\0\0\0\0\0\x41\0\0\0\x05\0\0\0\x40\0\0\0\x4d\0\0\0\
\x19\0\0\0\x80\0\0\0\x51\0\0\0\x08\0\0\0\xc0\0\0\0\xae\0\0\0\0\0\0\x0e\x26\0\0\
\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x29\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\
\x04\0\0\0\x05\0\0\0\0\0\0\0\x04\0\0\x04\x20\0\0\0\x3c\0\0\0\x28\0\0\0\0\0\0\0\
\x41\0\0\0\x05\0\0\0\x40\0\0\0\x4d\0\0\0\x19\0\0\0\x80\0\0\0\x51\0\0\0\x08\0\0\
\0\xc0\0\0\0\xbb\0\0\0\0\0\0\x0e\x2a\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x2d\0\0\
\0\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x0a\0\0\0\0\0\0\0\x04\0\0\x04\
\x20\0\0\0\x3c\0\0\0\x2c\0\0\0\0\0\0\0\x41\0\0\0\x05\0\0\0\x40\0\0\0\x4d\0\0\0\
\x19\0\0\0\x80\0\0\0\x51\0\0\0\x08\0\0\0\xc0\0\0\0\xc7\0\0\0\0\0\0\x0e\x2e\0\0\
\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x0a\0\0\0\0\0\0\0\x01\0\0\x0d\x02\0\0\0\xd7\0\0\
\0\x30\0\0\0\xdb\0\0\0\x01\0\0\x0c\x31\0\0\0\xe9\0\0\0\x13\0\0\x04\xc0\0\0\0\0\
\0\0\0\x34\0\0\0\0\0\0\0\xf8\0\0\0\x37\0\0\0\x80\0\0\0\xfe\0\0\0\x38\0\0\0\xc0\
\0\0\0\0\0\0\0\x3c\0\0\0\0\x01\0\0\x0b\x01\0\0\x3f\0\0\0\x40\x01\0\0\x12\x01\0\
\0\x42\0\0\0\x40\x02\0\0\x21\x01\0\0\x44\0\0\0\xc0\x02\0\0\x2a\x01\0\0\x45\0\0\
\0\0\x03\0\0\x31\x01\0\0\x36\0\0\0\x40\x03\0\0\x3a\x01\0\0\x47\0\0\0\x80\x03\0\
\0\x42\x01\0\0\x48\0\0\0\xc0\x03\0\0\x52\x01\0\0\x49\0\0\0\0\x04\0\0\x66\x01\0\
\0\x4f\0\0\0\x40\x04\0\0\x70\x01\0\0\x50\0\0\0\x80\x04\0\0\x7c\x01\0\0\x51\0\0\
\0\xc0\x04\0\0\x8f\x01\0\0\x36\0\0\0\0\x05\0\0\x9c\x01\0\0\x36\0\0\0\x40\x05\0\
\0\xa9\x01\0\0\x36\0\0\0\x80\x05\0\0\xb6\x01\0\0\x36\0\0\0\xc0\x05\0\0\0\0\0\0\
\x01\0\0\x05\x10\0\0\0\0\0\0\0\x35\0\0\0\0\0\0\0\0\0\0\0\x02\0\0\x04\x10\0\0\0\
\xc3\x01\0\0\x36\0\0\0\0\0\0\0\xcc\x01\0\0\x36\0\0\0\x40\0\0\0\xd3\x01\0\0\0\0\
\0\x01\x08\0\0\0\x40\0\0\0\0\0\0\0\0\0\0\x02\x53\0\0\0\xe1\x01\0\0\0\0\0\x08\
\x39\0\0\0\0\0\0\0\x01\0\0\x04\x08\0\0\0\xea\x01\0\0\x3a\0\0\0\0\0\0\0\xf1\x01\
\0\0\0\0\0\x08\x3b\0\0\0\xfa\x01\0\0\0\0\0\x08\x09\0\0\0\0\0\0\0\x02\0\0\x05\
\x08\0\0\0\xfe\x01\0\0\x3d\0\0\0\0\0\0\0\x07\x02\0\0\x3e\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\x0a\x3e\0\0\0\x12\x02\0\0\0\0\0\x08\x36\0\0\0\0\0\0\0\x02\0\0\x04\x20\0\
\0\0\x1d\x02\0\0\x40\0\0\0\0\0\0\0\x20\x02\0\0\x36\0\0\0\xc0\0\0\0\x30\x02\0\0\
\x03\0\0\x04\x18\0\0\0\x38\x02\0\0\x36\0\0\0\0\0\0\0\x4a\x02\0\0\x41\0\0\0\x40\
\0\0\0\x53\x02\0\0\x41\0\0\0\x80\0\0\0\0\0\0\0\0\0\0\x02\x40\0\0\0\x5b\x02\0\0\
\x02\0\0\x04\x10\0\0\0\x65\x02\0\0\x43\0\0\0\0\0\0\0\x6a\x02\0\0\x43\0\0\0\x40\
\0\0\0\0\0\0\0\0\0\0\x02\x42\0\0\0\0\0\0\0\0\0\0\x02\xdd\0\0\0\0\0\0\0\0\0\0\
\x02\x46\0\0\0\0\0\0\0\0\0\0\x0a\xd4\0\0\0\0\0\0\0\0\0\0\x02\x6c\0\0\0\0\0\0\0\
\0\0\0\x02\0\0\0\0\x6f\x02\0\0\0\0\0\x08\x4a\0\0\0\x7d\x02\0\0\0\0\0\x08\x4b\0\
\0\0\0\0\0\0\x01\0\0\x04\x08\0\0\0\x88\x02\0\0\x4c\0\0\0\0\0\0\0\x90\x02\0\0\0\
\0\0\x08\x4d\0\0\0\x94\x02\0\0\0\0\0\x08\x4e\0\0\0\x9a\x02\0\0\0\0\0\x01\x08\0\
\0\0\x40\0\0\x01\0\0\0\0\0\0\0\x02\xd5\0\0\0\0\0\0\0\0\0\0\x02\xde\0\0\0\x7c\
\x01\0\0\x01\0\0\x04\x08\0\0\0\xd7\0\0\0\x52\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x02\
\xdf\0\0\0\xa4\x02\0\0\x07\0\0\x04\x58\x04\0\0\0\0\0\0\x54\0\0\0\0\0\0\0\x8f\
\x01\0\0\x36\0\0\0\x80\x21\0\0\x9c\x01\0\0\x36\0\0\0\xc0\x21\0\0\xa9\x01\0\0\
\x36\0\0\0\0\x22\0\0\xb6\x01\0\0\x36\0\0\0\x40\x22\0\0\xae\x02\0\0\x36\0\0\0\
\x80\x22\0\0\xbb\x02\0\0\xb5\0\0\0\xc0\x22\0\0\0\0\0\0\x3a\0\0\x04\x30\x04\0\0\
\xc6\x02\0\0\x55\0\0\0\0\0\0\0\xcc\x02\0\0\x6a\0\0\0\xc0\0\0\0\xde\x02\0\0\x36\
\0\0\0\0\x01\0\0\xe8\x02\0\0\x36\0\0\0\x40\x01\0\0\xf9\x02\0\0\x36\0\0\0\x80\
\x01\0\0\x03\x03\0\0\x95\0\0\0\xc0\x01\0\0\x07\x03\0\0\x5e\0\0\0\0\x02\0\0\x18\
\x03\0\0\x5e\0\0\0\x20\x02\0\0\x21\x03\0\0\x5e\0\0\0\x40\x02\0\0\x2a\x03\0\0\
\x49\0\0\0\x80\x02\0\0\x39\x03\0\0\x02\0\0\0\xc0\x02\0\0\x43\x03\0\0\x57\0\0\0\
\xe0\x02\0\0\x53\x03\0\0\x99\0\0\0\0\x03\0\0\x5d\x03\0\0\x42\0\0\0\x40\x04\0\0\
\x64\x03\0\0\x36\0\0\0\xc0\x04\0\0\x70\x03\0\0\x36\0\0\0\0\x05\0\0\x7b\x03\0\0\
\x36\0\0\0\x40\x05\0\0\x84\x03\0\0\x36\0\0\0\x80\x05\0\0\x8e\x03\0\0\x4a\0\0\0\
\xc0\x05\0\0\x98\x03\0\0\x36\0\0\0\0\x06\0\0\xa0\x03\0\0\x36\0\0\0\x40\x06\0\0\
\xa8\x03\0\0\x36\0\0\0\x80\x06\0\0\xb1\x03\0\0\x36\0\0\0\xc0\x06\0\0\xbb\x03\0\
\0\x9a\0\0\0\0\x07\0\0\xcd\x03\0\0\x57\0\0\0\x20\x07\0\0\xd6\x03\0\0\x36\0\0\0\
\x40\x07\0\0\xe1\x03\0\0\x36\0\0\0\x80\x07\0\0\xea\x03\0\0\x36\0\0\0\xc0\x07\0\
\0\xf5\x03\0\0\x36\0\0\0\0\x08\0\0\xfe\x03\0\0\x36\0\0\0\x40\x08\0\0\x08\x04\0\
\0\x36\0\0\0\x80\x08\0\0\x0c\x04\0\0\x36\0\0\0\xc0\x08\0\0\x18\x04\0\0\x36\0\0\
\0\0\x09\0\0\x22\x04\0\0\x36\0\0\0\x40\x09\0\0\x2a\x04\0\0\x36\0\0\0\x80\x09\0\
\0\x34\x04\0\0\x36\0\0\0\xc0\x09\0\0\x3c\x04\0\0\x9c\0\0\0\0\x0a\0\0\x47\x04\0\
\0\xa1\0\0\0\x80\x15\0\0\x50\x04\0\0\xa2\0\0\0\x80\x1a\0\0\x57\x04\0\0\xa3\0\0\
\0\xc0\x1a\0\0\x5f\x04\0\0\x36\0\0\0\xc0\x1b\0\0\x65\x04\0\0\x57\0\0\0\0\x1c\0\
\0\x70\x04\0\0\xa7\0\0\0\x40\x1c\0\0\x7c\x04\0\0\xa8\0\0\0\x80\x1c\0\0\x82\x04\
\0\0\xa9\0\0\0\xc0\x1c\0\0\x8a\x04\0\0\x47\0\0\0\0\x1d\0\0\x93\x04\0\0\xaa\0\0\
\0\x40\x1d\0\0\xaa\x04\0\0\x36\0\0\0\x80\x1d\0\0\xb9\x04\0\0\x36\0\0\0\xc0\x1d\
\0\0\xca\x04\0\0\x02\0\0\0\0\x1e\0\0\xd8\x04\0\0\x5e\0\0\0\x20\x1e\0\0\xea\x04\
\0\0\xab\0\0\0\x40\x1e\0\0\xf8\x04\0\0\x49\0\0\0\x80\x1e\0\0\x06\x05\0\0\xad\0\
\0\0\xc0\x1e\0\0\x15\x05\0\0\xb2\0\0\0\xc0\x1f\0\0\x1e\x05\0\0\x36\0\0\0\0\x20\
\0\0\x30\x05\0\0\x36\0\0\0\x40\x20\0\0\x3f\x05\0\0\xb3\0\0\0\x80\x20\0\0\x47\
\x05\0\0\x03\0\0\x04\x18\0\0\0\0\0\0\0\x56\0\0\0\0\0\0\0\x52\x05\0\0\x48\0\0\0\
\x40\0\0\0\x5a\x05\0\0\x1b\0\0\0\x80\0\0\0\0\0\0\0\x02\0\0\x05\x04\0\0\0\x63\
\x05\0\0\x57\0\0\0\0\0\0\0\x6b\x05\0\0\x68\0\0\0\0\0\0\0\x7c\x05\0\0\0\0\0\x08\
\x58\0\0\0\x87\x05\0\0\x01\0\0\x04\x04\0\0\0\0\0\0\0\x59\0\0\0\0\0\0\0\0\0\0\0\
\x01\0\0\x05\x04\0\0\0\x90\x05\0\0\x5a\0\0\0\0\0\0\0\x96\x05\0\0\x01\0\0\x04\
\x04\0\0\0\xa3\x05\0\0\x5b\0\0\0\0\0\0\0\xac\x05\0\0\0\0\0\x08\x5c\0\0\0\xbc\
\x05\0\0\x01\0\0\x04\x04\0\0\0\0\0\0\0\x5d\0\0\0\0\0\0\0\0\0\0\0\x03\0\0\x05\
\x04\0\0\0\xc6\x05\0\0\x5e\0\0\0\0\0\0\0\0\0\0\0\x60\0\0\0\0\0\0\0\0\0\0\0\x64\
\0\0\0\0\0\0\0\xca\x05\0\0\0\0\0\x08\x5f\0\0\0\0\0\0\0\x01\0\0\x04\x04\0\0\0\
\x88\x02\0\0\x02\0\0\0\0\0\0\0\0\0\0\0\x02\0\0\x04\x02\0\0\0\xd3\x05\0\0\x61\0\
\0\0\0\0\0\0\xda\x05\0\0\x61\0\0\0\x08\0\0\0\xe2\x05\0\0\0\0\0\x08\x62\0\0\0\
\xe5\x05\0\0\0\0\0\x08\x63\0\0\0\xea\x05\0\0\0\0\0\x01\x01\0\0\0\x08\0\0\0\0\0\
\0\0\x02\0\0\x04\x04\0\0\0\xf8\x05\0\0\x65\0\0\0\0\0\0\0\x07\x06\0\0\x65\0\0\0\
\x10\0\0\0\x0c\x06\0\0\0\0\0\x08\x66\0\0\0\x10\x06\0\0\0\0\0\x08\x67\0\0\0\x16\
\x06\0\0\0\0\0\x01\x02\0\0\0\x10\0\0\0\x25\x06\0\0\0\0\0\x08\x69\0\0\0\0\0\0\0\
\0\0\0\x04\0\0\0\0\0\0\0\0\0\0\0\x02\x6b\0\0\0\0\0\0\0\x05\0\0\x0d\x36\0\0\0\0\
\0\0\0\x47\0\0\0\0\0\0\0\x36\0\0\0\0\0\0\0\x36\0\0\0\0\0\0\0\x36\0\0\0\0\0\0\0\
\x36\0\0\0\x33\x06\0\0\x14\0\0\x04\xe8\0\0\0\x38\x06\0\0\x6d\0\0\0\0\0\0\0\x3c\
\x06\0\0\x74\0\0\0\x80\0\0\0\x43\x06\0\0\x77\0\0\0\0\x01\0\0\x4b\x06\0\0\x78\0\
\0\0\x40\x01\0\0\x50\x06\0\0\x57\0\0\0\x80\x01\0\0\x57\x06\0\0\x49\0\0\0\xc0\
\x01\0\0\x5f\x06\0\0\x1b\0\0\0\0\x02\0\0\x67\x06\0\0\x7a\0\0\0\x20\x02\0\0\x6e\
\x06\0\0\x7b\0\0\0\x40\x02\0\0\x79\x06\0\0\x7e\0\0\0\x40\x03\0\0\x7f\x06\0\0\
\x80\0\0\0\x80\x03\0\0\x87\x06\0\0\x8e\0\0\0\x80\x04\0\0\x8e\x06\0\0\x90\0\0\0\
\xc0\x04\0\0\x93\x06\0\0\x3b\0\0\0\xc0\x05\0\0\x9d\x06\0\0\x48\0\0\0\0\x06\0\0\
\xa8\x06\0\0\x48\0\0\0\x40\x06\0\0\xb5\x06\0\0\x91\0\0\0\x80\x06\0\0\xba\x06\0\
\0\x92\0\0\0\xc0\x06\0\0\xc4\x06\0\0\x93\0\0\0\0\x07\0\0\xcd\x06\0\0\x93\0\0\0\
\x20\x07\0\0\0\0\0\0\x02\0\0\x05\x10\0\0\0\xd6\x06\0\0\x6e\0\0\0\0\0\0\0\xdf\
\x06\0\0\x70\0\0\0\0\0\0\0\xea\x06\0\0\x01\0\0\x04\x08\0\0\0\x65\x02\0\0\x6f\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\x02\x6e\0\0\0\xf5\x06\0\0\x02\0\0\x04\x10\0\0\0\x65\
\x02\0\0\x71\0\0\0\0\0\0\0\x03\x07\0\0\x72\0\0\0\x40\0\0\0\0\0\0\0\0\0\0\x02\
\x70\0\0\0\0\0\0\0\0\0\0\x02\x73\0\0\0\0\0\0\0\x01\0\0\x0d\0\0\0\0\0\0\0\0\x71\
\0\0\0\x08\x07\0\0\x02\0\0\x04\x10\0\0\0\x0d\x07\0\0\x75\0\0\0\0\0\0\0\x11\x07\
\0\0\x76\0\0\0\x40\0\0\0\0\0\0\0\0\0\0\x02\xcc\0\0\0\0\0\0\0\0\0\0\x02\xcd\0\0\
\0\0\0\0\0\0\0\0\x02\xcb\0\0\0\0\0\0\0\0\0\0\x02\x79\0\0\0\0\0\0\0\0\0\0\x0a\
\xd3\0\0\0\x18\x07\0\0\0\0\0\x08\x1b\0\0\0\x20\x07\0\0\x04\0\0\x04\x20\0\0\0\
\x7c\x04\0\0\x49\0\0\0\0\0\0\0\x26\x07\0\0\x7c\0\0\0\x40\0\0\0\x30\x07\0\0\x7d\
\0\0\0\x60\0\0\0\x34\x07\0\0\x42\0\0\0\x80\0\0\0\x3e\x07\0\0\0\0\0\x08\x5a\0\0\
\0\x4d\x07\0\0\x01\0\0\x04\x04\0\0\0\x07\x06\0\0\x5e\0\0\0\0\0\0\0\x63\x07\0\0\
\0\0\0\x08\x7f\0\0\0\x6a\x07\0\0\0\0\0\x08\x4e\0\0\0\x7a\x07\0\0\x06\0\0\x04\
\x20\0\0\0\x86\x07\0\0\x81\0\0\0\0\0\0\0\x8b\x07\0\0\x88\0\0\0\x40\0\0\0\x8f\
\x07\0\0\x89\0\0\0\x80\0\0\0\x98\x07\0\0\x8a\0\0\0\xa0\0\0\0\x9c\x07\0\0\x8a\0\
\0\0\xc0\0\0\0\xa1\x07\0\0\x02\0\0\0\xe0\0\0\0\xa8\x07\0\0\0\0\0\x08\x82\0\0\0\
\0\0\0\0\x01\0\0\x04\x08\0\0\0\xa3\x05\0\0\x83\0\0\0\0\0\0\0\xb1\x07\0\0\0\0\0\
\x08\x84\0\0\0\xbf\x07\0\0\x02\0\0\x04\x08\0\0\0\0\0\0\0\x85\0\0\0\0\0\0\0\x26\
\x07\0\0\x5b\0\0\0\x20\0\0\0\0\0\0\0\x02\0\0\x05\x04\0\0\0\xc7\x07\0\0\x5e\0\0\
\0\0\0\0\0\0\0\0\0\x86\0\0\0\0\0\0\0\0\0\0\0\x02\0\0\x04\x04\0\0\0\xcc\x07\0\0\
\x61\0\0\0\0\0\0\0\xd4\x07\0\0\x87\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\
\x61\0\0\0\x04\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\x02\xd7\0\0\0\x8f\x07\0\0\x05\0\0\
\x06\x04\0\0\0\xdd\x07\0\0\0\0\0\0\xe9\x07\0\0\x01\0\0\0\xf6\x07\0\0\x02\0\0\0\
\x03\x08\0\0\x03\0\0\0\x0f\x08\0\0\x04\0\0\0\x1b\x08\0\0\0\0\0\x08\x8b\0\0\0\0\
\0\0\0\x01\0\0\x04\x04\0\0\0\xc6\x05\0\0\x8c\0\0\0\0\0\0\0\x22\x08\0\0\0\0\0\
\x08\x8d\0\0\0\x28\x08\0\0\0\0\0\x08\x1b\0\0\0\0\0\0\0\0\0\0\x02\x8f\0\0\0\0\0\
\0\0\0\0\0\x0a\xd6\0\0\0\x39\x08\0\0\x06\0\0\x04\x20\0\0\0\x47\x08\0\0\x36\0\0\
\0\0\0\0\0\x4d\x08\0\0\x1b\0\0\0\x40\0\0\0\x52\x08\0\0\x1b\0\0\0\x60\0\0\0\x5d\
\x08\0\0\x1b\0\0\0\x80\0\0\0\x66\x08\0\0\x1b\0\0\0\xa0\0\0\0\x70\x08\0\0\x7e\0\
\0\0\xc0\0\0\0\0\0\0\0\0\0\0\x02\xd1\0\0\0\0\0\0\0\0\0\0\x02\xd0\0\0\0\x79\x08\
\0\0\0\0\0\x08\x94\0\0\0\x82\x08\0\0\0\0\0\x08\x1a\0\0\0\0\0\0\0\0\0\0\x02\x96\
\0\0\0\x86\x08\0\0\0\0\0\x08\x97\0\0\0\0\0\0\0\x01\0\0\x04\x08\0\0\0\x03\x03\0\
\0\x98\0\0\0\0\0\0\0\x8c\x08\0\0\0\0\0\x08\x3b\0\0\0\x95\x08\0\0\x05\0\0\x04\
\x28\0\0\0\xa2\x08\0\0\x49\0\0\0\0\0\0\0\x7c\x04\0\0\x49\0\0\0\x40\0\0\0\x30\
\x07\0\0\x7d\0\0\0\x80\0\0\0\x26\x07\0\0\x7c\0\0\0\xa0\0\0\0\x34\x07\0\0\x42\0\
\0\0\xc0\0\0\0\xa8\x08\0\0\0\0\0\x08\x9b\0\0\0\xb3\x08\0\0\x01\0\0\x04\x04\0\0\
\0\xbc\x08\0\0\x1b\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\x36\0\0\0\x04\0\0\0\
\x2e\0\0\0\xc5\x08\0\0\x04\0\0\x04\x28\0\0\0\x86\x07\0\0\x7c\0\0\0\0\0\0\0\xa2\
\x08\0\0\x4c\0\0\0\x40\0\0\0\xd4\x08\0\0\x42\0\0\0\x80\0\0\0\xd9\x08\0\0\x9e\0\
\0\0\0\x01\0\0\0\0\0\0\0\0\0\x02\x9f\0\0\0\xe2\x08\0\0\0\0\0\x08\xa0\0\0\0\xe6\
\x08\0\0\0\0\0\x08\x02\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\x9d\0\0\0\x04\0\0\0\x04\
\0\0\0\0\0\0\0\0\0\0\x02\xd8\0\0\0\xec\x08\0\0\0\0\0\x08\xa4\0\0\0\0\0\0\0\x04\
\0\0\x04\x20\0\0\0\xf9\x08\0\0\x4a\0\0\0\0\0\0\0\xfc\x08\0\0\xa5\0\0\0\x40\0\0\
\0\x03\x09\0\0\x48\0\0\0\x80\0\0\0\x5f\x04\0\0\x36\0\0\0\xc0\0\0\0\x08\x09\0\0\
\0\0\0\x08\xa6\0\0\0\x13\x09\0\0\x01\0\0\x04\x04\0\0\0\x23\x09\0\0\x5e\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\x02\xd9\0\0\0\0\0\0\0\0\0\0\x02\xcf\0\0\0\0\0\0\0\0\0\0\
\x02\xce\0\0\0\0\0\0\0\0\0\0\x02\xda\0\0\0\xea\x04\0\0\x01\0\0\x04\x08\0\0\0\
\x28\x09\0\0\xac\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x02\xdb\0\0\0\x31\x09\0\0\x03\0\0\
\x04\x20\0\0\0\x3d\x09\0\0\x49\0\0\0\0\0\0\0\x42\x09\0\0\x42\0\0\0\x40\0\0\0\
\x03\x07\0\0\xae\0\0\0\xc0\0\0\0\x48\x09\0\0\0\0\0\x08\xaf\0\0\0\0\0\0\0\0\0\0\
\x02\xb0\0\0\0\0\0\0\0\x01\0\0\x0d\0\0\0\0\0\0\0\0\xb1\0\0\0\0\0\0\0\0\0\0\x02\
\xad\0\0\0\0\0\0\0\0\0\0\x02\xdc\0\0\0\0\0\0\0\x03\0\0\x04\x20\0\0\0\xd4\x08\0\
\0\x42\0\0\0\0\0\0\0\x54\x09\0\0\x36\0\0\0\x80\0\0\0\x5b\x09\0\0\xb4\0\0\0\xc0\
\0\0\0\0\0\0\0\0\0\0\x02\xd2\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\x36\0\0\0\x04\0\0\
\0\0\0\0\0\x61\x09\0\0\x01\0\0\x0c\x31\0\0\0\x70\x09\0\0\x1a\0\0\x84\xa0\x01\0\
\0\x7d\x09\0\0\xb8\0\0\0\0\0\0\0\x81\x09\0\0\x36\0\0\0\x40\0\0\0\x8b\x09\0\0\
\x37\0\0\0\x80\0\0\0\x8e\x09\0\0\x36\0\0\0\xc0\0\0\0\x90\x09\0\0\x36\0\0\0\0\
\x01\0\0\x97\x09\0\0\x1b\0\0\0\x40\x01\0\x01\xa3\x09\0\0\x1b\0\0\0\x41\x01\0\
\x01\xb0\x09\0\0\x1b\0\0\0\x42\x01\0\x01\xbb\x09\0\0\x1b\0\0\0\x43\x01\0\x01\
\xce\x09\0\0\x47\0\0\0\x80\x01\0\0\xd9\x09\0\0\x47\0\0\0\xc0\x01\0\0\x33\x06\0\
\0\x47\0\0\0\0\x02\0\0\xe5\x09\0\0\xb9\0\0\0\x40\x02\0\0\xea\x09\0\0\x02\0\0\0\
\x80\x02\0\0\xf1\x09\0\0\x1b\0\0\0\xa0\x02\0\0\xfb\x09\0\0\x02\0\0\0\xc0\x02\0\
\0\0\x0a\0\0\x02\0\0\0\xe0\x02\0\0\x05\x0a\0\0\xba\0\0\0\0\x03\0\0\x0e\x0a\0\0\
\xba\0\0\0\x40\x03\0\0\x15\x0a\0\0\xba\0\0\0\x80\x03\0\0\x1c\x0a\0\0\x1b\0\0\0\
\xc0\x03\0\0\x29\x0a\0\0\x02\0\0\0\xe0\x03\0\0\x30\x0a\0\0\x36\0\0\0\0\x04\0\0\
\x37\x0a\0\0\x36\0\0\0\x40\x04\0\0\x3c\x0a\0\0\xbd\0\0\0\x80\x04\0\0\x47\x0a\0\
\0\xbf\0\0\0\0\x05\0\0\0\0\0\0\0\0\0\x02\x33\0\0\0\0\0\0\0\0\0\0\x02\xd6\0\0\0\
\0\0\0\0\0\0\0\x02\xbb\0\0\0\0\0\0\0\0\0\0\x0a\xbc\0\0\0\x4b\x0a\0\0\0\0\0\x01\
\x01\0\0\0\x08\0\0\x01\x50\x0a\0\0\x02\0\0\x04\x10\0\0\0\x57\x0a\0\0\xbe\0\0\0\
\0\0\0\0\x60\x0a\0\0\xbe\0\0\0\x40\0\0\0\x69\x0a\0\0\0\0\0\x08\x36\0\0\0\0\0\0\
\0\0\0\0\x03\0\0\0\0\xbc\0\0\0\x04\0\0\0\0\x01\0\0\x7a\x0a\0\0\x01\0\0\x0c\x31\
\0\0\0\x89\x0a\0\0\x01\0\0\x0c\x31\0\0\0\xa0\x0a\0\0\x01\0\0\x04\x08\0\0\0\xb0\
\x0a\0\0\x36\0\0\0\0\0\0\0\xb8\x0a\0\0\x04\0\0\x04\x10\x01\0\0\xc5\x0a\0\0\xc4\
\0\0\0\0\0\0\0\xca\x0a\0\0\x09\0\0\0\xc0\x07\0\0\xcd\x0a\0\0\x09\0\0\0\0\x08\0\
\0\xd0\x0a\0\0\x09\0\0\0\x40\x08\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\x09\0\0\0\x04\0\
\0\0\x1f\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\xbc\0\0\0\x04\0\0\0\x04\0\0\0\xd7\x0a\
\0\0\0\0\0\x0e\xc5\0\0\0\x01\0\0\0\xe0\x0a\0\0\0\0\0\x0e\x02\0\0\0\x01\0\0\0\
\xee\x0a\0\0\0\0\0\x0e\x02\0\0\0\x01\0\0\0\xfd\x0a\0\0\0\0\0\x0e\x02\0\0\0\x01\
\0\0\0\x08\x0b\0\0\0\0\0\x0e\x02\0\0\0\x01\0\0\0\x12\x0b\0\0\0\0\0\x07\0\0\0\0\
\x18\x0b\0\0\0\0\0\x07\0\0\0\0\x11\x07\0\0\0\0\0\x07\0\0\0\0\x21\x0b\0\0\0\0\0\
\x07\0\0\0\0\x30\x0b\0\0\0\0\0\x07\0\0\0\0\x3c\x0b\0\0\0\0\0\x07\0\0\0\0\x4a\
\x0b\0\0\0\0\0\x07\0\0\0\0\x55\x0b\0\0\0\0\0\x07\0\0\0\0\x60\x0b\0\0\0\0\0\x07\
\0\0\0\0\x70\x0b\0\0\0\0\0\x07\0\0\0\0\x85\x0b\0\0\0\0\0\x07\0\0\0\0\xe5\x09\0\
\0\0\0\0\x07\0\0\0\0\x8b\x07\0\0\0\0\0\x07\0\0\0\0\x8f\x0b\0\0\0\0\0\x07\0\0\0\
\0\x9c\x0b\0\0\0\0\0\x07\0\0\0\0\xa9\x0b\0\0\0\0\0\x07\0\0\0\0\x28\x09\0\0\0\0\
\0\x07\0\0\0\0\xc4\x0b\0\0\0\0\0\x07\0\0\0\0\x21\x01\0\0\0\0\0\x07\0\0\0\0\xd2\
\x0b\0\0\0\0\0\x07\0\0\0\0\xe2\x0b\0\0\0\0\0\x07\0\0\0\0\x4c\x12\0\0\x09\0\0\
\x0f\x28\x01\0\0\x0c\0\0\0\0\0\0\0\x20\0\0\0\x14\0\0\0\x20\0\0\0\x28\0\0\0\x18\
\0\0\0\x48\0\0\0\x20\0\0\0\x1d\0\0\0\x68\0\0\0\x20\0\0\0\x1f\0\0\0\x88\0\0\0\
\x20\0\0\0\x23\0\0\0\xa8\0\0\0\x20\0\0\0\x27\0\0\0\xc8\0\0\0\x20\0\0\0\x2b\0\0\
\0\xe8\0\0\0\x20\0\0\0\x2f\0\0\0\x08\x01\0\0\x20\0\0\0\x52\x12\0\0\x01\0\0\x0f\
\x04\0\0\0\xc6\0\0\0\0\0\0\0\x04\0\0\0\x5a\x12\0\0\x04\0\0\x0f\x10\0\0\0\xc7\0\
\0\0\0\0\0\0\x04\0\0\0\xc8\0\0\0\x04\0\0\0\x04\0\0\0\xc9\0\0\0\x08\0\0\0\x04\0\
\0\0\xca\0\0\0\x0c\0\0\0\x04\0\0\0\0\x69\x6e\x74\0\x5f\x5f\x41\x52\x52\x41\x59\
\x5f\x53\x49\x5a\x45\x5f\x54\x59\x50\x45\x5f\x5f\0\x5f\x5f\x75\x36\x34\0\x75\
\x6e\x73\x69\x67\x6e\x65\x64\x20\x6c\x6f\x6e\x67\x20\x6c\x6f\x6e\x67\0\x69\x6e\
\x6e\x65\x72\x5f\x6d\x61\x70\0\x74\x79\x70\x65\0\x6d\x61\x78\x5f\x65\x6e\x74\
\x72\x69\x65\x73\0\x6b\x65\x79\0\x76\x61\x6c\x75\x65\0\x6f\x75\x74\x65\x72\x5f\
\x61\x72\x72\0\x6b\x65\x79\x5f\x73\x69\x7a\x65\0\x76\x61\x6c\x75\x65\x5f\x73\
\x69\x7a\x65\0\x76\x61\x6c\x75\x65\x73\0\x6f\x75\x74\x65\x72\x5f\x68\x61\x73\
\x68\0\x5f\x5f\x75\x33\x32\0\x75\x6e\x73\x69\x67\x6e\x65\x64\x20\x69\x6e\x74\0\
\x61\x72\x72\x61\x79\0\x68\x61\x73\x68\0\x6c\x72\x75\x5f\x68\x61\x73\x68\0\x70\
\x65\x72\x63\x70\x75\x5f\x61\x72\x72\x61\x79\0\x70\x65\x72\x63\x70\x75\x5f\x68\
\x61\x73\x68\0\x6c\x72\x75\x5f\x70\x65\x72\x63\x70\x75\x5f\x68\x61\x73\x68\0\
\x63\x74\x78\0\x74\x65\x73\x74\x5f\x69\x6e\x74\x5f\x68\x6f\x6f\x6b\0\x76\x6d\
\x5f\x61\x72\x65\x61\x5f\x73\x74\x72\x75\x63\x74\0\x76\x6d\x5f\x6d\x6d\0\x76\
\x6d\x5f\x70\x61\x67\x65\x5f\x70\x72\x6f\x74\0\x73\x68\x61\x72\x65\x64\0\x61\
\x6e\x6f\x6e\x5f\x76\x6d\x61\x5f\x63\x68\x61\x69\x6e\0\x61\x6e\x6f\x6e\x5f\x76\
\x6d\x61\0\x76\x6d\x5f\x6f\x70\x73\0\x76\x6d\x5f\x70\x67\x6f\x66\x66\0\x76\x6d\
\x5f\x66\x69\x6c\x65\0\x76\x6d\x5f\x70\x72\x69\x76\x61\x74\x65\x5f\x64\x61\x74\
\x61\0\x73\x77\x61\x70\x5f\x72\x65\x61\x64\x61\x68\x65\x61\x64\x5f\x69\x6e\x66\
\x6f\0\x76\x6d\x5f\x70\x6f\x6c\x69\x63\x79\0\x6e\x75\x6d\x61\x62\x5f\x73\x74\
\x61\x74\x65\0\x76\x6d\x5f\x75\x73\x65\x72\x66\x61\x75\x6c\x74\x66\x64\x5f\x63\
\x74\x78\0\x72\x68\x5f\x72\x65\x73\x65\x72\x76\x65\x64\x31\0\x72\x68\x5f\x72\
\x65\x73\x65\x72\x76\x65\x64\x32\0\x72\x68\x5f\x72\x65\x73\x65\x72\x76\x65\x64\
\x33\0\x72\x68\x5f\x72\x65\x73\x65\x72\x76\x65\x64\x34\0\x76\x6d\x5f\x73\x74\
\x61\x72\x74\0\x76\x6d\x5f\x65\x6e\x64\0\x75\x6e\x73\x69\x67\x6e\x65\x64\x20\
\x6c\x6f\x6e\x67\0\x70\x67\x70\x72\x6f\x74\x5f\x74\0\x70\x67\x70\x72\x6f\x74\0\
\x70\x74\x65\x76\x61\x6c\x5f\x74\0\x75\x36\x34\0\x76\x6d\x5f\x66\x6c\x61\x67\
\x73\0\x5f\x5f\x76\x6d\x5f\x66\x6c\x61\x67\x73\0\x76\x6d\x5f\x66\x6c\x61\x67\
\x73\x5f\x74\0\x72\x62\0\x72\x62\x5f\x73\x75\x62\x74\x72\x65\x65\x5f\x6c\x61\
\x73\x74\0\x72\x62\x5f\x6e\x6f\x64\x65\0\x5f\x5f\x72\x62\x5f\x70\x61\x72\x65\
\x6e\x74\x5f\x63\x6f\x6c\x6f\x72\0\x72\x62\x5f\x72\x69\x67\x68\x74\0\x72\x62\
\x5f\x6c\x65\x66\x74\0\x6c\x69\x73\x74\x5f\x68\x65\x61\x64\0\x6e\x65\x78\x74\0\
\x70\x72\x65\x76\0\x61\x74\x6f\x6d\x69\x63\x5f\x6c\x6f\x6e\x67\x5f\x74\0\x61\
\x74\x6f\x6d\x69\x63\x36\x34\x5f\x74\0\x63\x6f\x75\x6e\x74\x65\x72\0\x73\x36\
\x34\0\x5f\x5f\x73\x36\x34\0\x6c\x6f\x6e\x67\x20\x6c\x6f\x6e\x67\0\x6d\x6d\x5f\
\x73\x74\x72\x75\x63\x74\0\x72\x68\x5f\x72\x65\x73\x65\x72\x76\x65\x64\x35\0\
\x63\x70\x75\x5f\x62\x69\x74\x6d\x61\x70\0\x6d\x6d\x5f\x6d\x74\0\x67\x65\x74\
\x5f\x75\x6e\x6d\x61\x70\x70\x65\x64\x5f\x61\x72\x65\x61\0\x6d\x6d\x61\x70\x5f\
\x62\x61\x73\x65\0\x6d\x6d\x61\x70\x5f\x6c\x65\x67\x61\x63\x79\x5f\x62\x61\x73\
\x65\0\x74\x61\x73\x6b\x5f\x73\x69\x7a\x65\0\x70\x67\x64\0\x6d\x65\x6d\x62\x61\
\x72\x72\x69\x65\x72\x5f\x73\x74\x61\x74\x65\0\x6d\x6d\x5f\x75\x73\x65\x72\x73\
\0\x6d\x6d\x5f\x63\x6f\x75\x6e\x74\0\x70\x67\x74\x61\x62\x6c\x65\x73\x5f\x62\
\x79\x74\x65\x73\0\x6d\x61\x70\x5f\x63\x6f\x75\x6e\x74\0\x70\x61\x67\x65\x5f\
\x74\x61\x62\x6c\x65\x5f\x6c\x6f\x63\x6b\0\x6d\x6d\x61\x70\x5f\x6c\x6f\x63\x6b\
\0\x6d\x6d\x6c\x69\x73\x74\0\x68\x69\x77\x61\x74\x65\x72\x5f\x72\x73\x73\0\x68\
\x69\x77\x61\x74\x65\x72\x5f\x76\x6d\0\x74\x6f\x74\x61\x6c\x5f\x76\x6d\0\x6c\
\x6f\x63\x6b\x65\x64\x5f\x76\x6d\0\x70\x69\x6e\x6e\x65\x64\x5f\x76\x6d\0\x64\
\x61\x74\x61\x5f\x76\x6d\0\x65\x78\x65\x63\x5f\x76\x6d\0\x73\x74\x61\x63\x6b\
\x5f\x76\x6d\0\x64\x65\x66\x5f\x66\x6c\x61\x67\x73\0\x77\x72\x69\x74\x65\x5f\
\x70\x72\x6f\x74\x65\x63\x74\x5f\x73\x65\x71\0\x61\x72\x67\x5f\x6c\x6f\x63\x6b\
\0\x73\x74\x61\x72\x74\x5f\x63\x6f\x64\x65\0\x65\x6e\x64\x5f\x63\x6f\x64\x65\0\
\x73\x74\x61\x72\x74\x5f\x64\x61\x74\x61\0\x65\x6e\x64\x5f\x64\x61\x74\x61\0\
\x73\x74\x61\x72\x74\x5f\x62\x72\x6b\0\x62\x72\x6b\0\x73\x74\x61\x72\x74\x5f\
\x73\x74\x61\x63\x6b\0\x61\x72\x67\x5f\x73\x74\x61\x72\x74\0\x61\x72\x67\x5f\
\x65\x6e\x64\0\x65\x6e\x76\x5f\x73\x74\x61\x72\x74\0\x65\x6e\x76\x5f\x65\x6e\
\x64\0\x73\x61\x76\x65\x64\x5f\x61\x75\x78\x76\0\x72\x73\x73\x5f\x73\x74\x61\
\x74\0\x62\x69\x6e\x66\x6d\x74\0\x63\x6f\x6e\x74\x65\x78\x74\0\x66\x6c\x61\x67\
\x73\0\x69\x6f\x63\x74\x78\x5f\x6c\x6f\x63\x6b\0\x69\x6f\x63\x74\x78\x5f\x74\
\x61\x62\x6c\x65\0\x6f\x77\x6e\x65\x72\0\x75\x73\x65\x72\x5f\x6e\x73\0\x65\x78\
\x65\x5f\x66\x69\x6c\x65\0\x6e\x6f\x74\x69\x66\x69\x65\x72\x5f\x73\x75\x62\x73\
\x63\x72\x69\x70\x74\x69\x6f\x6e\x73\0\x6e\x75\x6d\x61\x5f\x6e\x65\x78\x74\x5f\
\x73\x63\x61\x6e\0\x6e\x75\x6d\x61\x5f\x73\x63\x61\x6e\x5f\x6f\x66\x66\x73\x65\
\x74\0\x6e\x75\x6d\x61\x5f\x73\x63\x61\x6e\x5f\x73\x65\x71\0\x74\x6c\x62\x5f\
\x66\x6c\x75\x73\x68\x5f\x70\x65\x6e\x64\x69\x6e\x67\0\x75\x70\x72\x6f\x62\x65\
\x73\x5f\x73\x74\x61\x74\x65\0\x68\x75\x67\x65\x74\x6c\x62\x5f\x75\x73\x61\x67\
\x65\0\x61\x73\x79\x6e\x63\x5f\x70\x75\x74\x5f\x77\x6f\x72\x6b\0\x69\x6f\x6d\
\x6d\x75\x5f\x6d\x6d\0\x6b\x73\x6d\x5f\x6d\x65\x72\x67\x69\x6e\x67\x5f\x70\x61\
\x67\x65\x73\0\x6b\x73\x6d\x5f\x72\x6d\x61\x70\x5f\x69\x74\x65\x6d\x73\0\x6c\
\x72\x75\x5f\x67\x65\x6e\0\x6d\x61\x70\x6c\x65\x5f\x74\x72\x65\x65\0\x6d\x61\
\x5f\x72\x6f\x6f\x74\0\x6d\x61\x5f\x66\x6c\x61\x67\x73\0\x6d\x61\x5f\x6c\x6f\
\x63\x6b\0\x6d\x61\x5f\x65\x78\x74\x65\x72\x6e\x61\x6c\x5f\x6c\x6f\x63\x6b\0\
\x73\x70\x69\x6e\x6c\x6f\x63\x6b\x5f\x74\0\x73\x70\x69\x6e\x6c\x6f\x63\x6b\0\
\x72\x6c\x6f\x63\x6b\0\x72\x61\x77\x5f\x73\x70\x69\x6e\x6c\x6f\x63\x6b\0\x72\
\x61\x77\x5f\x6c\x6f\x63\x6b\0\x61\x72\x63\x68\x5f\x73\x70\x69\x6e\x6c\x6f\x63\
\x6b\x5f\x74\0\x71\x73\x70\x69\x6e\x6c\x6f\x63\x6b\0\x76\x61\x6c\0\x61\x74\x6f\
\x6d\x69\x63\x5f\x74\0\x6c\x6f\x63\x6b\x65\x64\0\x70\x65\x6e\x64\x69\x6e\x67\0\
\x75\x38\0\x5f\x5f\x75\x38\0\x75\x6e\x73\x69\x67\x6e\x65\x64\x20\x63\x68\x61\
\x72\0\x6c\x6f\x63\x6b\x65\x64\x5f\x70\x65\x6e\x64\x69\x6e\x67\0\x74\x61\x69\
\x6c\0\x75\x31\x36\0\x5f\x5f\x75\x31\x36\0\x75\x6e\x73\x69\x67\x6e\x65\x64\x20\
\x73\x68\x6f\x72\x74\0\x6c\x6f\x63\x6b\x64\x65\x70\x5f\x6d\x61\x70\x5f\x70\0\
\x66\x69\x6c\x65\0\x66\x5f\x75\0\x66\x5f\x70\x61\x74\x68\0\x66\x5f\x69\x6e\x6f\
\x64\x65\0\x66\x5f\x6f\x70\0\x66\x5f\x6c\x6f\x63\x6b\0\x66\x5f\x63\x6f\x75\x6e\
\x74\0\x66\x5f\x66\x6c\x61\x67\x73\0\x66\x5f\x6d\x6f\x64\x65\0\x66\x5f\x70\x6f\
\x73\x5f\x6c\x6f\x63\x6b\0\x66\x5f\x70\x6f\x73\0\x66\x5f\x6f\x77\x6e\x65\x72\0\
\x66\x5f\x63\x72\x65\x64\0\x66\x5f\x72\x61\0\x66\x5f\x76\x65\x72\x73\x69\x6f\
\x6e\0\x66\x5f\x73\x65\x63\x75\x72\x69\x74\x79\0\x70\x72\x69\x76\x61\x74\x65\
\x5f\x64\x61\x74\x61\0\x66\x5f\x65\x70\0\x66\x5f\x6d\x61\x70\x70\x69\x6e\x67\0\
\x66\x5f\x77\x62\x5f\x65\x72\x72\0\x66\x5f\x73\x62\x5f\x65\x72\x72\0\x66\x75\
\x5f\x6c\x6c\x69\x73\x74\0\x66\x75\x5f\x72\x63\x75\x68\x65\x61\x64\0\x6c\x6c\
\x69\x73\x74\x5f\x6e\x6f\x64\x65\0\x63\x61\x6c\x6c\x62\x61\x63\x6b\x5f\x68\x65\
\x61\x64\0\x66\x75\x6e\x63\0\x70\x61\x74\x68\0\x6d\x6e\x74\0\x64\x65\x6e\x74\
\x72\x79\0\x66\x6d\x6f\x64\x65\x5f\x74\0\x6d\x75\x74\x65\x78\0\x77\x61\x69\x74\
\x5f\x6c\x6f\x63\x6b\0\x6f\x73\x71\0\x77\x61\x69\x74\x5f\x6c\x69\x73\x74\0\x72\
\x61\x77\x5f\x73\x70\x69\x6e\x6c\x6f\x63\x6b\x5f\x74\0\x6f\x70\x74\x69\x6d\x69\
\x73\x74\x69\x63\x5f\x73\x70\x69\x6e\x5f\x71\x75\x65\x75\x65\0\x6c\x6f\x66\x66\
\x5f\x74\0\x5f\x5f\x6b\x65\x72\x6e\x65\x6c\x5f\x6c\x6f\x66\x66\x5f\x74\0\x66\
\x6f\x77\x6e\x5f\x73\x74\x72\x75\x63\x74\0\x6c\x6f\x63\x6b\0\x70\x69\x64\0\x70\
\x69\x64\x5f\x74\x79\x70\x65\0\x75\x69\x64\0\x65\x75\x69\x64\0\x73\x69\x67\x6e\
\x75\x6d\0\x72\x77\x6c\x6f\x63\x6b\x5f\x74\0\x61\x72\x63\x68\x5f\x72\x77\x6c\
\x6f\x63\x6b\x5f\x74\0\x71\x72\x77\x6c\x6f\x63\x6b\0\x63\x6e\x74\x73\0\x77\x6c\
\x6f\x63\x6b\x65\x64\0\x5f\x5f\x6c\x73\x74\x61\x74\x65\0\x50\x49\x44\x54\x59\
\x50\x45\x5f\x50\x49\x44\0\x50\x49\x44\x54\x59\x50\x45\x5f\x54\x47\x49\x44\0\
\x50\x49\x44\x54\x59\x50\x45\x5f\x50\x47\x49\x44\0\x50\x49\x44\x54\x59\x50\x45\
\x5f\x53\x49\x44\0\x50\x49\x44\x54\x59\x50\x45\x5f\x4d\x41\x58\0\x6b\x75\x69\
\x64\x5f\x74\0\x75\x69\x64\x5f\x74\0\x5f\x5f\x6b\x65\x72\x6e\x65\x6c\x5f\x75\
\x69\x64\x33\x32\x5f\x74\0\x66\x69\x6c\x65\x5f\x72\x61\x5f\x73\x74\x61\x74\x65\
\0\x73\x74\x61\x72\x74\0\x73\x69\x7a\x65\0\x61\x73\x79\x6e\x63\x5f\x73\x69\x7a\
\x65\0\x72\x61\x5f\x70\x61\x67\x65\x73\0\x6d\x6d\x61\x70\x5f\x6d\x69\x73\x73\0\
\x70\x72\x65\x76\x5f\x70\x6f\x73\0\x65\x72\x72\x73\x65\x71\x5f\x74\0\x75\x33\
\x32\0\x70\x67\x64\x5f\x74\0\x70\x67\x64\x76\x61\x6c\x5f\x74\0\x72\x77\x5f\x73\
\x65\x6d\x61\x70\x68\x6f\x72\x65\0\x63\x6f\x75\x6e\x74\0\x73\x65\x71\x63\x6f\
\x75\x6e\x74\x5f\x74\0\x73\x65\x71\x63\x6f\x75\x6e\x74\0\x73\x65\x71\x75\x65\
\x6e\x63\x65\0\x70\x65\x72\x63\x70\x75\x5f\x63\x6f\x75\x6e\x74\x65\x72\0\x6c\
\x69\x73\x74\0\x63\x6f\x75\x6e\x74\x65\x72\x73\0\x73\x33\x32\0\x5f\x5f\x73\x33\
\x32\0\x6d\x6d\x5f\x63\x6f\x6e\x74\x65\x78\x74\x5f\x74\0\x69\x64\0\x70\x69\x6e\
\x6e\x65\x64\0\x76\x64\x73\x6f\0\x72\x65\x66\x63\x6f\x75\x6e\x74\x5f\x74\0\x72\
\x65\x66\x63\x6f\x75\x6e\x74\x5f\x73\x74\x72\x75\x63\x74\0\x72\x65\x66\x73\0\
\x78\x6f\x6c\x5f\x61\x72\x65\x61\0\x77\x6f\x72\x6b\x5f\x73\x74\x72\x75\x63\x74\
\0\x64\x61\x74\x61\0\x65\x6e\x74\x72\x79\0\x77\x6f\x72\x6b\x5f\x66\x75\x6e\x63\
\x5f\x74\0\x62\x69\x74\x6d\x61\x70\0\x6d\x65\x6d\x63\x67\0\x74\x65\x73\x74\x5f\
\x76\x6f\x69\x64\x5f\x68\x6f\x6f\x6b\0\x6c\x69\x6e\x75\x78\x5f\x62\x69\x6e\x70\
\x72\x6d\0\x76\x6d\x61\0\x76\x6d\x61\x5f\x70\x61\x67\x65\x73\0\x6d\x6d\0\x70\0\
\x61\x72\x67\x6d\x69\x6e\0\x68\x61\x76\x65\x5f\x65\x78\x65\x63\x66\x64\0\x65\
\x78\x65\x63\x66\x64\x5f\x63\x72\x65\x64\x73\0\x73\x65\x63\x75\x72\x65\x65\x78\
\x65\x63\0\x70\x6f\x69\x6e\x74\x5f\x6f\x66\x5f\x6e\x6f\x5f\x72\x65\x74\x75\x72\
\x6e\0\x65\x78\x65\x63\x75\x74\x61\x62\x6c\x65\0\x69\x6e\x74\x65\x72\x70\x72\
\x65\x74\x65\x72\0\x63\x72\x65\x64\0\x75\x6e\x73\x61\x66\x65\0\x70\x65\x72\x5f\
\x63\x6c\x65\x61\x72\0\x61\x72\x67\x63\0\x65\x6e\x76\x63\0\x66\x69\x6c\x65\x6e\
\x61\x6d\x65\0\x69\x6e\x74\x65\x72\x70\0\x66\x64\x70\x61\x74\x68\0\x69\x6e\x74\
\x65\x72\x70\x5f\x66\x6c\x61\x67\x73\0\x65\x78\x65\x63\x66\x64\0\x6c\x6f\x61\
\x64\x65\x72\0\x65\x78\x65\x63\0\x72\x6c\x69\x6d\x5f\x73\x74\x61\x63\x6b\0\x62\
\x75\x66\0\x63\x68\x61\x72\0\x72\x6c\x69\x6d\x69\x74\0\x72\x6c\x69\x6d\x5f\x63\
\x75\x72\0\x72\x6c\x69\x6d\x5f\x6d\x61\x78\0\x5f\x5f\x6b\x65\x72\x6e\x65\x6c\
\x5f\x75\x6c\x6f\x6e\x67\x5f\x74\0\x74\x65\x73\x74\x5f\x74\x61\x73\x6b\x5f\x66\
\x72\x65\x65\0\x74\x65\x73\x74\x5f\x73\x79\x73\x5f\x73\x65\x74\x64\x6f\x6d\x61\
\x69\x6e\x6e\x61\x6d\x65\0\x70\x74\x5f\x72\x65\x67\x73\x5f\x5f\x5f\x61\x72\x6d\
\x36\x34\0\x6f\x72\x69\x67\x5f\x78\x30\0\x75\x73\x65\x72\x5f\x70\x74\x5f\x72\
\x65\x67\x73\0\x72\x65\x67\x73\0\x73\x70\0\x70\x63\0\x70\x73\x74\x61\x74\x65\0\
\x5f\x6c\x69\x63\x65\x6e\x73\x65\0\x6d\x6f\x6e\x69\x74\x6f\x72\x65\x64\x5f\x70\
\x69\x64\0\x6d\x70\x72\x6f\x74\x65\x63\x74\x5f\x63\x6f\x75\x6e\x74\0\x62\x70\
\x72\x6d\x5f\x63\x6f\x75\x6e\x74\0\x63\x6f\x70\x79\x5f\x74\x65\x73\x74\0\x69\
\x6e\x6f\x64\x65\0\x76\x66\x73\x6d\x6f\x75\x6e\x74\0\x75\x73\x65\x72\x5f\x6e\
\x61\x6d\x65\x73\x70\x61\x63\x65\0\x74\x61\x73\x6b\x5f\x73\x74\x72\x75\x63\x74\
\0\x61\x64\x64\x72\x65\x73\x73\x5f\x73\x70\x61\x63\x65\0\x68\x6c\x69\x73\x74\
\x5f\x68\x65\x61\x64\0\x6d\x65\x6d\x5f\x63\x67\x72\x6f\x75\x70\0\x66\x69\x6c\
\x65\x5f\x6f\x70\x65\x72\x61\x74\x69\x6f\x6e\x73\0\x76\x6d\x5f\x6f\x70\x65\x72\
\x61\x74\x69\x6f\x6e\x73\x5f\x73\x74\x72\x75\x63\x74\0\x6d\x65\x6d\x70\x6f\x6c\
\x69\x63\x79\0\x6c\x69\x6e\x75\x78\x5f\x62\x69\x6e\x66\x6d\x74\0\x6b\x69\x6f\
\x63\x74\x78\x5f\x74\x61\x62\x6c\x65\0\x6d\x6d\x75\x5f\x6e\x6f\x74\x69\x66\x69\
\x65\x72\x5f\x73\x75\x62\x73\x63\x72\x69\x70\x74\x69\x6f\x6e\x73\0\x69\x6f\x6d\
\x6d\x75\x5f\x6d\x6d\x5f\x64\x61\x74\x61\0\x76\x6d\x61\x5f\x6e\x75\x6d\x61\x62\
\x5f\x73\x74\x61\x74\x65\0\x75\x73\x65\x72\x66\x61\x75\x6c\x74\x66\x64\x5f\x63\
\x74\x78\0\x2f\x62\x75\x69\x6c\x64\x64\x69\x72\x2f\x62\x75\x69\x6c\x64\x2f\x42\
\x55\x49\x4c\x44\x2f\x6b\x65\x72\x6e\x65\x6c\x2d\x35\x2e\x31\x34\x2e\x30\x2d\
\x35\x31\x31\x2e\x72\x70\x69\x36\x34\x2e\x65\x6c\x39\x2f\x6c\x69\x6e\x75\x78\
\x2d\x35\x2e\x31\x34\x2e\x30\x2d\x35\x31\x31\x2e\x72\x70\x69\x36\x34\x2e\x65\
\x6c\x39\x2e\x61\x61\x72\x63\x68\x36\x34\x2f\x74\x6f\x6f\x6c\x73\x2f\x74\x65\
\x73\x74\x69\x6e\x67\x2f\x73\x65\x6c\x66\x74\x65\x73\x74\x73\x2f\x62\x70\x66\
\x2f\x70\x72\x6f\x67\x73\x2f\x6c\x73\x6d\x2e\x63\0\x69\x6e\x74\x20\x42\x50\x46\
\x5f\x50\x52\x4f\x47\x28\x74\x65\x73\x74\x5f\x69\x6e\x74\x5f\x68\x6f\x6f\x6b\
\x2c\x20\x73\x74\x72\x75\x63\x74\x20\x76\x6d\x5f\x61\x72\x65\x61\x5f\x73\x74\
\x72\x75\x63\x74\x20\x2a\x76\x6d\x61\x2c\0\x09\x69\x66\x20\x28\x72\x65\x74\x20\
\x21\x3d\x20\x30\x29\0\x09\x5f\x5f\x73\x33\x32\x20\x70\x69\x64\x20\x3d\x20\x62\
\x70\x66\x5f\x67\x65\x74\x5f\x63\x75\x72\x72\x65\x6e\x74\x5f\x70\x69\x64\x5f\
\x74\x67\x69\x64\x28\x29\x20\x3e\x3e\x20\x33\x32\x3b\0\x09\x69\x73\x5f\x73\x74\
\x61\x63\x6b\x20\x3d\x20\x28\x76\x6d\x61\x2d\x3e\x76\x6d\x5f\x73\x74\x61\x72\
\x74\x20\x3c\x3d\x20\x76\x6d\x61\x2d\x3e\x76\x6d\x5f\x6d\x6d\x2d\x3e\x73\x74\
\x61\x72\x74\x5f\x73\x74\x61\x63\x6b\x20\x26\x26\0\x09\x09\x20\x20\x20\x20\x76\
\x6d\x61\x2d\x3e\x76\x6d\x5f\x65\x6e\x64\x20\x3e\x3d\x20\x76\x6d\x61\x2d\x3e\
\x76\x6d\x5f\x6d\x6d\x2d\x3e\x73\x74\x61\x72\x74\x5f\x73\x74\x61\x63\x6b\x29\
\x3b\0\x09\x69\x66\x20\x28\x69\x73\x5f\x73\x74\x61\x63\x6b\x20\x26\x26\x20\x6d\
\x6f\x6e\x69\x74\x6f\x72\x65\x64\x5f\x70\x69\x64\x20\x3d\x3d\x20\x70\x69\x64\
\x29\x20\x7b\0\x09\x09\x6d\x70\x72\x6f\x74\x65\x63\x74\x5f\x63\x6f\x75\x6e\x74\
\x2b\x2b\x3b\0\x69\x6e\x74\x20\x42\x50\x46\x5f\x50\x52\x4f\x47\x28\x74\x65\x73\
\x74\x5f\x76\x6f\x69\x64\x5f\x68\x6f\x6f\x6b\x2c\x20\x73\x74\x72\x75\x63\x74\
\x20\x6c\x69\x6e\x75\x78\x5f\x62\x69\x6e\x70\x72\x6d\x20\x2a\x62\x70\x72\x6d\
\x29\0\x09\x5f\x5f\x75\x33\x32\x20\x70\x69\x64\x20\x3d\x20\x62\x70\x66\x5f\x67\
\x65\x74\x5f\x63\x75\x72\x72\x65\x6e\x74\x5f\x70\x69\x64\x5f\x74\x67\x69\x64\
\x28\x29\x20\x3e\x3e\x20\x33\x32\x3b\0\x09\x5f\x5f\x75\x33\x32\x20\x6b\x65\x79\
\x20\x3d\x20\x30\x3b\0\x09\x69\x66\x20\x28\x6d\x6f\x6e\x69\x74\x6f\x72\x65\x64\
\x5f\x70\x69\x64\x20\x3d\x3d\x20\x70\x69\x64\x29\0\x09\x09\x62\x70\x72\x6d\x5f\
\x63\x6f\x75\x6e\x74\x2b\x2b\x3b\0\x09\x62\x70\x66\x5f\x63\x6f\x70\x79\x5f\x66\
\x72\x6f\x6d\x5f\x75\x73\x65\x72\x28\x61\x72\x67\x73\x2c\x20\x73\x69\x7a\x65\
\x6f\x66\x28\x61\x72\x67\x73\x29\x2c\x20\x28\x76\x6f\x69\x64\x20\x2a\x29\x62\
\x70\x72\x6d\x2d\x3e\x76\x6d\x61\x2d\x3e\x76\x6d\x5f\x6d\x6d\x2d\x3e\x61\x72\
\x67\x5f\x73\x74\x61\x72\x74\x29\x3b\0\x09\x62\x70\x66\x5f\x63\x6f\x70\x79\x5f\
\x66\x72\x6f\x6d\x5f\x75\x73\x65\x72\x28\x61\x72\x67\x73\x2c\x20\x73\x69\x7a\
\x65\x6f\x66\x28\x61\x72\x67\x73\x29\x2c\x20\x28\x76\x6f\x69\x64\x20\x2a\x29\
\x62\x70\x72\x6d\x2d\x3e\x6d\x6d\x2d\x3e\x61\x72\x67\x5f\x73\x74\x61\x72\x74\
\x29\x3b\0\x09\x76\x61\x6c\x75\x65\x20\x3d\x20\x62\x70\x66\x5f\x6d\x61\x70\x5f\
\x6c\x6f\x6f\x6b\x75\x70\x5f\x65\x6c\x65\x6d\x28\x26\x61\x72\x72\x61\x79\x2c\
\x20\x26\x6b\x65\x79\x29\x3b\0\x09\x69\x66\x20\x28\x76\x61\x6c\x75\x65\x29\0\
\x09\x09\x2a\x76\x61\x6c\x75\x65\x20\x3d\x20\x30\x3b\0\x09\x76\x61\x6c\x75\x65\
\x20\x3d\x20\x62\x70\x66\x5f\x6d\x61\x70\x5f\x6c\x6f\x6f\x6b\x75\x70\x5f\x65\
\x6c\x65\x6d\x28\x26\x68\x61\x73\x68\x2c\x20\x26\x6b\x65\x79\x29\x3b\0\x09\x76\
\x61\x6c\x75\x65\x20\x3d\x20\x62\x70\x66\x5f\x6d\x61\x70\x5f\x6c\x6f\x6f\x6b\
\x75\x70\x5f\x65\x6c\x65\x6d\x28\x26\x6c\x72\x75\x5f\x68\x61\x73\x68\x2c\x20\
\x26\x6b\x65\x79\x29\x3b\0\x09\x76\x61\x6c\x75\x65\x20\x3d\x20\x62\x70\x66\x5f\
\x6d\x61\x70\x5f\x6c\x6f\x6f\x6b\x75\x70\x5f\x65\x6c\x65\x6d\x28\x26\x70\x65\
\x72\x63\x70\x75\x5f\x61\x72\x72\x61\x79\x2c\x20\x26\x6b\x65\x79\x29\x3b\0\x09\
\x76\x61\x6c\x75\x65\x20\x3d\x20\x62\x70\x66\x5f\x6d\x61\x70\x5f\x6c\x6f\x6f\
\x6b\x75\x70\x5f\x65\x6c\x65\x6d\x28\x26\x70\x65\x72\x63\x70\x75\x5f\x68\x61\
\x73\x68\x2c\x20\x26\x6b\x65\x79\x29\x3b\0\x09\x76\x61\x6c\x75\x65\x20\x3d\x20\
\x62\x70\x66\x5f\x6d\x61\x70\x5f\x6c\x6f\x6f\x6b\x75\x70\x5f\x65\x6c\x65\x6d\
\x28\x26\x6c\x72\x75\x5f\x70\x65\x72\x63\x70\x75\x5f\x68\x61\x73\x68\x2c\x20\
\x26\x6b\x65\x79\x29\x3b\0\x09\x69\x6e\x6e\x65\x72\x5f\x6d\x61\x70\x20\x3d\x20\
\x62\x70\x66\x5f\x6d\x61\x70\x5f\x6c\x6f\x6f\x6b\x75\x70\x5f\x65\x6c\x65\x6d\
\x28\x26\x6f\x75\x74\x65\x72\x5f\x61\x72\x72\x2c\x20\x26\x6b\x65\x79\x29\x3b\0\
\x09\x69\x66\x20\x28\x69\x6e\x6e\x65\x72\x5f\x6d\x61\x70\x29\x20\x7b\0\x09\x09\
\x76\x61\x6c\x75\x65\x20\x3d\x20\x62\x70\x66\x5f\x6d\x61\x70\x5f\x6c\x6f\x6f\
\x6b\x75\x70\x5f\x65\x6c\x65\x6d\x28\x69\x6e\x6e\x65\x72\x5f\x6d\x61\x70\x2c\
\x20\x26\x6b\x65\x79\x29\x3b\0\x09\x09\x69\x66\x20\x28\x76\x61\x6c\x75\x65\x29\
\0\x09\x09\x09\x2a\x76\x61\x6c\x75\x65\x20\x3d\x20\x30\x3b\0\x09\x69\x6e\x6e\
\x65\x72\x5f\x6d\x61\x70\x20\x3d\x20\x62\x70\x66\x5f\x6d\x61\x70\x5f\x6c\x6f\
\x6f\x6b\x75\x70\x5f\x65\x6c\x65\x6d\x28\x26\x6f\x75\x74\x65\x72\x5f\x68\x61\
\x73\x68\x2c\x20\x26\x6b\x65\x79\x29\x3b\0\x69\x6e\x74\x20\x42\x50\x46\x5f\x50\
\x52\x4f\x47\x28\x74\x65\x73\x74\x5f\x74\x61\x73\x6b\x5f\x66\x72\x65\x65\x2c\
\x20\x73\x74\x72\x75\x63\x74\x20\x74\x61\x73\x6b\x5f\x73\x74\x72\x75\x63\x74\
\x20\x2a\x74\x61\x73\x6b\x29\0\x69\x6e\x74\x20\x42\x50\x46\x5f\x50\x52\x4f\x47\
\x28\x74\x65\x73\x74\x5f\x73\x79\x73\x5f\x73\x65\x74\x64\x6f\x6d\x61\x69\x6e\
\x6e\x61\x6d\x65\x2c\x20\x73\x74\x72\x75\x63\x74\x20\x70\x74\x5f\x72\x65\x67\
\x73\x20\x2a\x72\x65\x67\x73\x29\0\x09\x76\x6f\x69\x64\x20\x2a\x70\x74\x72\x20\
\x3d\x20\x28\x76\x6f\x69\x64\x20\x2a\x29\x50\x54\x5f\x52\x45\x47\x53\x5f\x50\
\x41\x52\x4d\x31\x5f\x53\x59\x53\x43\x41\x4c\x4c\x28\x72\x65\x67\x73\x29\x3b\0\
\x09\x69\x6e\x74\x20\x6c\x65\x6e\x20\x3d\x20\x50\x54\x5f\x52\x45\x47\x53\x5f\
\x50\x41\x52\x4d\x32\x5f\x53\x59\x53\x43\x41\x4c\x4c\x28\x72\x65\x67\x73\x29\
\x3b\0\x09\x69\x6e\x74\x20\x62\x75\x66\x20\x3d\x20\x30\x3b\0\x09\x72\x65\x74\
\x20\x3d\x20\x62\x70\x66\x5f\x63\x6f\x70\x79\x5f\x66\x72\x6f\x6d\x5f\x75\x73\
\x65\x72\x28\x26\x62\x75\x66\x2c\x20\x73\x69\x7a\x65\x6f\x66\x28\x62\x75\x66\
\x29\x2c\x20\x70\x74\x72\x29\x3b\0\x09\x69\x66\x20\x28\x6c\x65\x6e\x20\x3d\x3d\
\x20\x2d\x32\x20\x26\x26\x20\x72\x65\x74\x20\x3d\x3d\x20\x30\x20\x26\x26\x20\
\x62\x75\x66\x20\x3d\x3d\x20\x31\x32\x33\x34\x29\0\x09\x69\x66\x20\x28\x6c\x65\
\x6e\x20\x3d\x3d\x20\x2d\x33\x20\x26\x26\x20\x72\x65\x74\x20\x3d\x3d\x20\x2d\
\x45\x46\x41\x55\x4c\x54\x29\0\x30\x3a\x31\0\x30\x3a\x30\x3a\x30\x3a\x30\0\x30\
\x3a\x30\x3a\x33\x31\0\x30\x3a\x30\x3a\x30\x3a\x31\0\x30\x3a\x30\0\x30\x3a\x30\
\x3a\x33\x32\0\x30\x3a\x32\0\x30\x3a\x30\x3a\x31\0\x2e\x6d\x61\x70\x73\0\x6c\
\x69\x63\x65\x6e\x73\x65\0\x2e\x62\x73\x73\0\x6c\x73\x6d\x2f\x66\x69\x6c\x65\
\x5f\x6d\x70\x72\x6f\x74\x65\x63\x74\0\x6c\x73\x6d\x2e\x73\x2f\x62\x70\x72\x6d\
\x5f\x63\x6f\x6d\x6d\x69\x74\x74\x65\x64\x5f\x63\x72\x65\x64\x73\0\x6c\x73\x6d\
\x2f\x74\x61\x73\x6b\x5f\x66\x72\x65\x65\0\x66\x65\x6e\x74\x72\x79\x2e\x73\x2f\
\x5f\x5f\x61\x72\x6d\x36\x34\x5f\x73\x79\x73\x5f\x73\x65\x74\x64\x6f\x6d\x61\
\x69\x6e\x6e\x61\x6d\x65\0\0\0\0\0\0\0\0\x9f\xeb\x01\0\x20\0\0\0\0\0\0\0\x44\0\
\0\0\x44\0\0\0\xd4\x04\0\0\x18\x05\0\0\xec\0\0\0\x08\0\0\0\x5f\x12\0\0\x01\0\0\
\0\0\0\0\0\x32\0\0\0\x71\x12\0\0\x01\0\0\0\0\0\0\0\xb6\0\0\0\x8c\x12\0\0\x01\0\
\0\0\0\0\0\0\xc0\0\0\0\x9a\x12\0\0\x01\0\0\0\0\0\0\0\xc1\0\0\0\x10\0\0\0\x5f\
\x12\0\0\x10\0\0\0\0\0\0\0\xf2\x0b\0\0\x6f\x0c\0\0\x05\x64\x01\0\x08\0\0\0\xf2\
\x0b\0\0\xa7\x0c\0\0\x06\x70\x01\0\x10\0\0\0\xf2\x0b\0\0\x6f\x0c\0\0\x05\x64\
\x01\0\x18\0\0\0\xf2\x0b\0\0\xb6\x0c\0\0\x0e\x7c\x01\0\x30\0\0\0\xf2\x0b\0\0\
\xe5\x0c\0\0\x24\x88\x01\0\x38\0\0\0\xf2\x0b\0\0\xe5\x0c\0\0\x13\x88\x01\0\x40\
\0\0\0\xf2\x0b\0\0\xe5\x0c\0\0\x2b\x88\x01\0\x48\0\0\0\xf2\x0b\0\0\xe5\x0c\0\0\
\x37\x88\x01\0\x50\0\0\0\xf2\x0b\0\0\x1e\x0d\0\0\x1b\x8c\x01\0\x58\0\0\0\xf2\
\x0b\0\0\x1e\x0d\0\0\x22\x8c\x01\0\x60\0\0\0\xf2\x0b\0\0\x1e\x0d\0\0\x0c\x8c\
\x01\0\x68\0\0\0\xf2\x0b\0\0\x4d\x0d\0\0\x0f\x94\x01\0\x88\0\0\0\xf2\x0b\0\0\0\
\0\0\0\0\0\0\0\x90\0\0\0\xf2\x0b\0\0\x4d\x0d\0\0\x0f\x94\x01\0\x98\0\0\0\xf2\
\x0b\0\0\x76\x0d\0\0\x11\x98\x01\0\xc8\0\0\0\xf2\x0b\0\0\x6f\x0c\0\0\x05\x64\
\x01\0\x71\x12\0\0\x2d\0\0\0\0\0\0\0\xf2\x0b\0\0\x8a\x0d\0\0\x05\xb8\x01\0\x08\
\0\0\0\xf2\x0b\0\0\xc2\x0d\0\0\x0e\xc0\x01\0\x18\0\0\0\xf2\x0b\0\0\xf1\x0d\0\0\
\x08\xcc\x01\0\x20\0\0\0\xf2\x0b\0\0\x01\x0e\0\0\x06\xd8\x01\0\x38\0\0\0\xf2\
\x0b\0\0\xc2\x0d\0\0\x29\xc0\x01\0\x40\0\0\0\xf2\x0b\0\0\x01\x0e\0\0\x06\xd8\
\x01\0\x48\0\0\0\xf2\x0b\0\0\x1c\x0e\0\0\x0d\xdc\x01\0\x70\0\0\0\xf2\x0b\0\0\
\x2c\x0e\0\0\x37\xe4\x01\0\x78\0\0\0\xf2\x0b\0\0\x2c\x0e\0\0\x3c\xe4\x01\0\x80\
\0\0\0\xf2\x0b\0\0\x2c\x0e\0\0\x43\xe4\x01\0\x90\0\0\0\xf2\x0b\0\0\x2c\x0e\0\0\
\x37\xe4\x01\0\x98\0\0\0\xf2\x0b\0\0\x2c\x0e\0\0\x02\xe4\x01\0\xb0\0\0\0\xf2\
\x0b\0\0\x7a\x0e\0\0\x37\xe8\x01\0\xb8\0\0\0\xf2\x0b\0\0\x7a\x0e\0\0\x3b\xe8\
\x01\0\xc0\0\0\0\xf2\x0b\0\0\x7a\x0e\0\0\x02\xe8\x01\0\xe0\0\0\0\xf2\x0b\0\0\
\x2c\x0e\0\0\x37\xe4\x01\0\xe8\0\0\0\xf2\x0b\0\0\xc0\x0e\0\0\x0a\xf0\x01\0\0\
\x01\0\0\xf2\x0b\0\0\xec\x0e\0\0\x06\xf4\x01\0\x10\x01\0\0\xf2\x0b\0\0\xf8\x0e\
\0\0\x0a\xf8\x01\0\x20\x01\0\0\xf2\x0b\0\0\x06\x0f\0\0\x0a\xfc\x01\0\x40\x01\0\
\0\xf2\x0b\0\0\xec\x0e\0\0\x06\0\x02\0\x50\x01\0\0\xf2\x0b\0\0\xf8\x0e\0\0\x0a\
\x04\x02\0\x60\x01\0\0\xf2\x0b\0\0\x31\x0f\0\0\x0a\x08\x02\0\x80\x01\0\0\xf2\
\x0b\0\0\xec\x0e\0\0\x06\x0c\x02\0\x90\x01\0\0\xf2\x0b\0\0\xf8\x0e\0\0\x0a\x10\
\x02\0\xa0\x01\0\0\xf2\x0b\0\0\x60\x0f\0\0\x0a\x14\x02\0\xc0\x01\0\0\xf2\x0b\0\
\0\xec\x0e\0\0\x06\x18\x02\0\xd0\x01\0\0\xf2\x0b\0\0\xf8\x0e\0\0\x0a\x1c\x02\0\
\xe0\x01\0\0\xf2\x0b\0\0\x93\x0f\0\0\x0a\x20\x02\0\0\x02\0\0\xf2\x0b\0\0\xec\
\x0e\0\0\x06\x24\x02\0\x10\x02\0\0\xf2\x0b\0\0\xf8\x0e\0\0\x0a\x28\x02\0\x20\
\x02\0\0\xf2\x0b\0\0\xc5\x0f\0\0\x0a\x2c\x02\0\x40\x02\0\0\xf2\x0b\0\0\xec\x0e\
\0\0\x06\x30\x02\0\x50\x02\0\0\xf2\x0b\0\0\xf8\x0e\0\0\x0a\x34\x02\0\x60\x02\0\
\0\xf2\x0b\0\0\xfb\x0f\0\0\x0e\x38\x02\0\x80\x02\0\0\xf2\x0b\0\0\x2f\x10\0\0\
\x06\x3c\x02\0\x90\x02\0\0\xf2\x0b\0\0\x41\x10\0\0\x0b\x40\x02\0\xa8\x02\0\0\
\xf2\x0b\0\0\x71\x10\0\0\x07\x44\x02\0\xb8\x02\0\0\xf2\x0b\0\0\x7e\x10\0\0\x0b\
\x48\x02\0\xc8\x02\0\0\xf2\x0b\0\0\x8d\x10\0\0\x0e\x50\x02\0\xe8\x02\0\0\xf2\
\x0b\0\0\x2f\x10\0\0\x06\x54\x02\0\xf8\x02\0\0\xf2\x0b\0\0\x41\x10\0\0\x0b\x58\
\x02\0\x10\x03\0\0\xf2\x0b\0\0\x71\x10\0\0\x07\x5c\x02\0\x20\x03\0\0\xf2\x0b\0\
\0\x7e\x10\0\0\x0b\x60\x02\0\x28\x03\0\0\xf2\x0b\0\0\x8a\x0d\0\0\x05\xb8\x01\0\
\x8c\x12\0\0\x01\0\0\0\0\0\0\0\xf2\x0b\0\0\xc2\x10\0\0\x05\x78\x02\0\x9a\x12\0\
\0\x0d\0\0\0\0\0\0\0\xf2\x0b\0\0\xf9\x10\0\0\0\x98\x02\0\x08\0\0\0\xf2\x0b\0\0\
\xf9\x10\0\0\x05\x98\x02\0\x28\0\0\0\xf2\x0b\0\0\0\0\0\0\0\0\0\0\x30\0\0\0\xf2\
\x0b\0\0\x34\x11\0\0\x16\xa0\x02\0\x40\0\0\0\xf2\x0b\0\0\x34\x11\0\0\x16\xa0\
\x02\0\x48\0\0\0\xf2\x0b\0\0\x66\x11\0\0\x0c\xa4\x02\0\x58\0\0\0\xf2\x0b\0\0\
\x8e\x11\0\0\x06\xa8\x02\0\x68\0\0\0\xf2\x0b\0\0\0\0\0\0\0\0\0\0\x70\0\0\0\xf2\
\x0b\0\0\x9c\x11\0\0\x08\xb4\x02\0\x80\0\0\0\xf2\x0b\0\0\xcf\x11\0\0\x10\xb8\
\x02\0\xa0\0\0\0\xf2\x0b\0\0\xfa\x11\0\0\x10\xc0\x02\0\xb8\0\0\0\xf2\x0b\0\0\0\
\0\0\0\0\0\0\0\xe0\0\0\0\xf2\x0b\0\0\xf9\x10\0\0\x05\x98\x02\0\x10\0\0\0\x5f\
\x12\0\0\x06\0\0\0\x30\0\0\0\x33\0\0\0\x1c\x12\0\0\0\0\0\0\x38\0\0\0\x33\0\0\0\
\x20\x12\0\0\0\0\0\0\x40\0\0\0\x53\0\0\0\x28\x12\0\0\0\0\0\0\x50\0\0\0\x33\0\0\
\0\x1c\x12\0\0\0\0\0\0\x58\0\0\0\x53\0\0\0\x28\x12\0\0\0\0\0\0\x60\0\0\0\x33\0\
\0\0\x2f\x12\0\0\0\0\0\0\x71\x12\0\0\x05\0\0\0\x70\0\0\0\xb7\0\0\0\x37\x12\0\0\
\0\0\0\0\x78\0\0\0\x33\0\0\0\x1c\x12\0\0\0\0\0\0\x80\0\0\0\x53\0\0\0\x3b\x12\0\
\0\0\0\0\0\xb0\0\0\0\xb7\0\0\0\x42\x12\0\0\0\0\0\0\xb8\0\0\0\x53\0\0\0\x3b\x12\
\0\0\0\0\0\0\x9a\x12\0\0\x02\0\0\0\0\0\0\0\xc2\0\0\0\x37\x12\0\0\0\0\0\0\x48\0\
\0\0\xc3\0\0\0\x46\x12\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\x01\0\0\0\x03\0\0\0\x20\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x40\0\0\0\
\0\0\0\0\x36\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\x09\0\0\0\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x78\x02\0\0\0\0\0\0\x78\
\x03\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\x18\0\0\0\0\0\0\0\x11\0\0\
\0\x01\0\0\0\x06\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xf0\x05\0\0\0\0\0\0\xd0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x23\0\0\0\x01\0\0\0\
\x06\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xc0\x06\0\0\0\0\0\0\x38\x03\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x3e\0\0\0\x01\0\0\0\x06\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\xf8\x09\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x4c\0\0\0\x01\0\0\0\x06\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\x08\x0a\0\0\0\0\0\0\xf0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\x6f\0\0\0\x01\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xf8\
\x0a\0\0\0\0\0\0\x28\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\x75\0\0\0\x01\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x20\x0c\0\0\0\0\
\0\0\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x7d\0\
\0\0\x08\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x24\x0c\0\0\0\0\0\0\x10\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xc2\x01\0\0\x09\0\0\
\0\x40\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x28\x0c\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\x02\
\0\0\0\x03\0\0\0\x08\0\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\xd8\x01\0\0\x09\0\0\0\x40\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x48\x0c\0\0\0\0\0\0\xa0\0\0\0\0\0\0\0\x02\0\0\0\
\x04\0\0\0\x08\0\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\xf7\x01\0\0\x09\0\0\0\x40\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\xe8\x0c\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\x02\0\0\0\x06\0\
\0\0\x08\0\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\x1e\x02\0\0\x09\0\0\0\x40\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\xf8\x0c\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\x02\0\0\0\x07\0\0\0\
\x08\0\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\x28\x02\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\x18\x0d\0\0\0\0\0\0\xe1\x2b\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\x2d\x02\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\x39\0\0\0\0\0\0\x24\x06\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0";

	*sz = sizeof(data) - 1;
	return (const void *)data;
}

#ifdef __cplusplus
struct lsm *lsm::open(const struct bpf_object_open_opts *opts) { return lsm__open_opts(opts); }
struct lsm *lsm::open_and_load() { return lsm__open_and_load(); }
int lsm::load(struct lsm *skel) { return lsm__load(skel); }
int lsm::attach(struct lsm *skel) { return lsm__attach(skel); }
void lsm::detach(struct lsm *skel) { lsm__detach(skel); }
void lsm::destroy(struct lsm *skel) { lsm__destroy(skel); }
const void *lsm::elf_bytes(size_t *sz) { return lsm__elf_bytes(sz); }
#endif /* __cplusplus */

__attribute__((unused)) static void
lsm__assert(struct lsm *s __attribute__((unused)))
{
#ifdef __cplusplus
#define _Static_assert static_assert
#endif
	_Static_assert(sizeof(s->bss->monitored_pid) == 4, "unexpected size of 'monitored_pid'");
	_Static_assert(sizeof(s->bss->mprotect_count) == 4, "unexpected size of 'mprotect_count'");
	_Static_assert(sizeof(s->bss->bprm_count) == 4, "unexpected size of 'bprm_count'");
	_Static_assert(sizeof(s->bss->copy_test) == 4, "unexpected size of 'copy_test'");
#ifdef __cplusplus
#undef _Static_assert
#endif
}

#endif /* __LSM_SKEL_H__ */
